'use strict';
var fs = require('fs');
var util_1 = require('util');
function log(fmt) {
    var args = [];
    for (var _i = 1; _i < arguments.length; _i++) {
        args[_i - 1] = arguments[_i];
    }
    var cb = undefined;
    if (args.length && typeof args[args.length - 1] === 'function') {
        cb = args[args.length - 1];
        args = args.slice(0, -1);
    }
    var prog = process.argv[1].split('/').slice(-1);
    var msg = prog + ': ' + util_1.format.apply(undefined, [fmt].concat(args)) + '\n';
    if (cb)
        process.stderr.write(msg, cb);
    else
        process.stderr.write(msg);
}
function parseArgs(args, handlers, argsRequired) {
    if (argsRequired === void 0) { argsRequired = false; }
    var ok = true;
    var positionalArgs = args.filter(function (arg) { return arg.substring(0, 1) !== '-'; });
    args = args.filter(function (arg) { return arg.substring(0, 1) === '-'; });
    var errs = 0;
    function done() {
        errs--;
        if (!errs)
            process.exit(1);
    }
    function error() {
        var args = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            args[_i - 0] = arguments[_i];
        }
        errs++;
        ok = false;
        log.apply(this, args.concat([done]));
    }
    function usage() {
        errs++;
        var prog = process.argv[1].split('/').slice(-1);
        var flags = Object.keys(handlers).concat(['h']).sort().join('');
        var msg = util_1.format('usage: %s [-%s] ARGS\n', prog, flags);
        process.stderr.write(msg, done);
    }
    outer: for (var i = 0; i < args.length; i++) {
        var argList = args[i].slice(1);
        if (argList.length && argList[0] === '-') {
            error('unknown option "%s"', args[i]);
            continue;
        }
        for (var j = 0; j < argList.length; j++) {
            var arg = argList[j];
            if (handlers[arg]) {
                handlers[arg]();
            }
            else if (arg === 'h') {
                ok = false;
                break outer;
            }
            else {
                error('invalid option "%s"', arg);
            }
        }
    }
    if (!ok || (argsRequired && positionalArgs.length === 0))
        usage();
    return [positionalArgs, ok];
}
function main() {
    'use strict';
    var pathToScript = process.argv[1];
    var pflag = false;
    var _a = parseArgs(process.argv.slice(2), {
        'p': function () { return pflag = true; },
    }, true), args = _a[0], ok = _a[1];
    var code = 0;
    var completed = 0;
    function finished() {
        completed++;
        if (completed === args.length) {
            process.exit(code);
        }
    }
    function pmkdir(path) {
        var subdirs = path.split('/');
        var subpath = '';
        function makePath(index) {
            fs.mkdir(subpath, function (oerr) {
                if (oerr) {
                    log('fs.mkdir: %s', oerr, finished);
                }
                else {
                    if (index === subdirs.length) {
                        finished();
                    }
                    else {
                        subpath += subdirs[index] + '/';
                        makePath(index + 1);
                    }
                }
            });
        }
        function checkExists(index) {
            subpath += subdirs[index] + '/';
            fs.stat(subpath, function (oerr, stats) {
                if (oerr) {
                    if (oerr.code === "ENOENT")
                        makePath(index + 1);
                }
                else {
                    checkExists(index + 1);
                }
            });
        }
        checkExists(0);
    }
    args.map(function (path, i) {
        fs.mkdir(path, function (err) {
            if (err && pflag) {
                pmkdir(path);
            }
            else if (err) {
                code = 1;
                process.stderr.write(err.message + '\n', finished);
                return;
            }
            else {
                finished();
            }
        });
    });
}
main();
