'use strict';
var syscall_1 = require('../syscall');
var constants = require('./constants');
var marshal = require('node-binary-marshal');
var FSReqWrap = (function () {
    function FSReqWrap() {
        this.oncomplete = undefined;
        this.context = undefined;
    }
    FSReqWrap.prototype.complete = function () {
        var args = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            args[_i - 0] = arguments[_i];
        }
        this.oncomplete.apply(this, arguments);
    };
    return FSReqWrap;
}());
exports.FSReqWrap = FSReqWrap;
var Stats = (function () {
    function Stats() {
    }
    Stats.prototype._updateBirthtime = function () {
        var oldest = this.atime;
        if (this.mtime < oldest)
            oldest = this.mtime;
        if (this.ctime < oldest)
            oldest = this.ctime;
    };
    Stats.prototype._checkModeProperty = function (property) {
        return ((this.mode & constants.S_IFMT) === property);
    };
    Stats.prototype.isDirectory = function () {
        return this._checkModeProperty(constants.S_IFDIR);
    };
    Stats.prototype.isFile = function () {
        return this._checkModeProperty(constants.S_IFREG);
    };
    Stats.prototype.isBlockDevice = function () {
        return this._checkModeProperty(constants.S_IFBLK);
    };
    Stats.prototype.isCharacterDevice = function () {
        return this._checkModeProperty(constants.S_IFCHR);
    };
    Stats.prototype.isSymbolicLink = function () {
        return this._checkModeProperty(constants.S_IFLNK);
    };
    Stats.prototype.isFIFO = function () {
        return this._checkModeProperty(constants.S_IFIFO);
    };
    Stats.prototype.isSocket = function () {
        return this._checkModeProperty(constants.S_IFSOCK);
    };
    return Stats;
}());
exports.Stats = Stats;
function FSInitialize(stats) {
}
exports.FSInitialize = FSInitialize;
function open(path, flags, mode, req) {
    syscall_1.syscall.open(path, flags, mode, req.complete.bind(req));
}
exports.open = open;
function unlink(path, req) {
    syscall_1.syscall.unlink(path, req.complete.bind(req));
}
exports.unlink = unlink;
function rmdir(path, req) {
    syscall_1.syscall.rmdir(path, req.complete.bind(req));
}
exports.rmdir = rmdir;
function mkdir(path, mode, req) {
    syscall_1.syscall.mkdir(path, mode, req.complete.bind(req));
}
exports.mkdir = mkdir;
function utimes(path, atime, mtime, req) {
    syscall_1.syscall.utimes(path, atime, mtime, req.complete.bind(req));
}
exports.utimes = utimes;
function futimes(fd, atime, mtime, req) {
    syscall_1.syscall.futimes(fd, atime, mtime, req.complete.bind(req));
}
exports.futimes = futimes;
function readdir(path, req) {
    syscall_1.syscall.readdir(path, req.complete.bind(req));
}
exports.readdir = readdir;
function statFinished(req, err, buf) {
    if (err) {
        if (typeof err === 'number') {
            var nerr = err;
            err = new Error('FS error');
            if (nerr === -constants.ENOENT) {
                err.code = 'ENOENT';
            }
        }
        req.complete(err, null);
        return;
    }
    var stats = new Stats();
    var view = new DataView(buf.buffer, buf.byteOffset);
    var len;
    _a = marshal.Unmarshal(stats, view, 0, marshal.fs.StatDef), len = _a[0], err = _a[1];
    stats._updateBirthtime();
    req.complete(err, stats);
    var _a;
}
function fstat(fd, req) {
    syscall_1.syscall.fstat(fd, statFinished.bind(undefined, req));
}
exports.fstat = fstat;
function lstat(path, req) {
    syscall_1.syscall.lstat(path, statFinished.bind(undefined, req));
}
exports.lstat = lstat;
function stat(path, req) {
    syscall_1.syscall.stat(path, statFinished.bind(undefined, req));
}
exports.stat = stat;
function read(fd, buffer, offset, len, pos, req) {
    if (typeof pos === 'undefined')
        pos = -1;
    syscall_1.syscall.pread(fd, len, pos, function readFinished(err, lenRead, data) {
        if (err) {
            req.complete(err, null);
            return;
        }
        for (var i = 0; i < lenRead; i++)
            buffer.writeUInt8(data[i], offset + i);
        req.complete(null, lenRead);
    });
}
exports.read = read;
function writeBuffer(fd, buffer, offset, len, pos, req) {
    syscall_1.syscall.pwrite(fd, buffer.slice(offset, offset + len), pos, req.complete.bind(req));
}
exports.writeBuffer = writeBuffer;
function writeBuffers(fd, chunks, pos, req) {
    var errored = false;
    var done = 0;
    var written = 0;
    function chunkComplete(err, count) {
        done++;
        if (err && !errored) {
            req.complete(err, undefined);
            errored = true;
            return;
        }
        written += count;
        if (done === chunks.length)
            req.complete(null, written);
    }
    for (var i = 0; i < chunks.length; i++) {
        var chunkReq = new FSReqWrap();
        chunkReq.oncomplete = chunkComplete;
        writeBuffer(fd, chunks[i], 0, chunks[i].length, undefined, chunkReq);
    }
}
exports.writeBuffers = writeBuffers;
function close(fd, req) {
    syscall_1.syscall.close(fd, req.complete.bind(req));
}
exports.close = close;
