'use strict';
var syscall_1 = require('../syscall');
var uv = require('./uv');
var ERROR = uv.UV_EMFILE;
var ENOENT = uv.UV_ENOENT;
;
var Process = (function () {
    function Process() {
        this.onerror = undefined;
        this.onexit = undefined;
    }
    Process.prototype.spawn = function (opts) {
        var _this = this;
        var files = [];
        for (var i = 0; i < opts.stdio.length; i++) {
            var f = opts.stdio[i];
            if (f.type !== 'fd')
                throw new Error('unsupported type ' + f.type + ' for FD ' + f.fd);
            files.push(f.fd);
        }
        var bins = (process.env['PATH'] || '/usr/local/bin:/bin:/usr/bin').split(':');
        var cmds = [opts.file];
        if (cmds[0].indexOf('/') === -1) {
            cmds = bins.map(function (p) {
                if (p.length && p[p.length - 1] === '/')
                    p = p.slice(0, -1);
                return p + '/' + opts.file;
            });
        }
        var cwd = opts.cwd;
        if (!cwd)
            cwd = process.cwd();
        var trySpawn = function () {
            var cmd = cmds.shift();
            syscall_1.syscall.spawn(cwd, cmd, opts.args, opts.envPairs, files, function (err, pid) {
                if (err === -ENOENT && cmds.length) {
                    trySpawn();
                    return;
                }
                else if (err) {
                    if (_this.onexit)
                        _this.onexit(-128, -1);
                    return;
                }
                _this.pid = pid;
                syscall_1.syscall.addEventListener('child', _this.handleSigchild.bind(_this));
            });
        };
        trySpawn();
        return null;
    };
    Process.prototype.close = function () {
    };
    Process.prototype.ref = function () {
        console.log('TODO: Process.ref');
    };
    Process.prototype.unref = function () {
        console.log('TODO: Process.unref');
    };
    Process.prototype.handleSigchild = function (data) {
        var pid = data.args[0];
        if (pid !== this.pid) {
            return;
        }
        var exitCode = data.args[1];
        var signalCode = data.args[2];
        if (this.onexit)
            this.onexit(exitCode, signalCode);
    };
    return Process;
}());
exports.Process = Process;
