'use strict';
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
var uv = require('./uv');
var syscall_1 = require('../syscall');
var stream_wrap_1 = require('./stream_wrap');
var marshal = require('node-binary-marshal');
var Req = (function () {
    function Req() {
    }
    return Req;
}());
exports.Req = Req;
var TCPConnectWrap = (function () {
    function TCPConnectWrap(fd) {
        if (fd === void 0) { fd = -1; }
        this.oncomplete = undefined;
        this.address = '';
        this.port = -1;
        this.fd = fd;
    }
    return TCPConnectWrap;
}());
exports.TCPConnectWrap = TCPConnectWrap;
var TCP = (function (_super) {
    __extends(TCP, _super);
    function TCP(fd, bound) {
        if (fd === void 0) { fd = -1; }
        if (bound === void 0) { bound = false; }
        _super.call(this);
        this.bound = false;
        this.listenPending = false;
        this.backlog = 511;
        this.fd = fd;
        this.bound = bound;
    }
    TCP.prototype.getpeername = function (out) {
        out.address = this.addr;
        out.port = this.port;
        out.family = 'tcp';
    };
    TCP.prototype.close = function (cb) {
        var _this = this;
        syscall_1.syscall.close(this.fd, function () {
            _this.fd = -1;
            cb();
        });
    };
    TCP.prototype.connect = function (conn, addr, port) {
        var _this = this;
        syscall_1.syscall.socket(syscall_1.AF.INET, syscall_1.SOCK.STREAM, 0, function (err, fd) {
            if (err) {
                console.log('socket open failed');
                return;
            }
            var sockAddr = {
                family: syscall_1.SOCK.STREAM,
                port: port,
                addr: addr,
            };
            var buf = new Uint8Array(marshal.socket.SockAddrInDef.length);
            var view = new DataView(buf.buffer, buf.byteOffset);
            var _;
            _a = marshal.Marshal(view, 0, sockAddr, marshal.socket.SockAddrInDef), _ = _a[0], err = _a[1];
            if (err) {
                console.log('connect: marshal failed');
                return;
            }
            syscall_1.syscall.connect(fd, buf, function (connErr, localAddr, localPort) {
                var req = new Req();
                req.localAddress = localAddr;
                req.localPort = localPort;
                req.address = addr;
                req.port = port;
                if (connErr) {
                    conn.oncomplete(-1, _this, req, false, false);
                    return;
                }
                _this.fd = fd;
                _this.bound = true;
                conn.oncomplete(0, _this, req, true, true);
            });
            var _a;
        });
    };
    TCP.prototype.bind6 = function (ipAddr, port) {
        if (ipAddr === '::')
            return this.bind('localhost', port);
        throw new Error('TODO: implement bind6');
    };
    TCP.prototype.bind = function (ipAddr, port) {
        var _this = this;
        syscall_1.syscall.socket(syscall_1.AF.INET, syscall_1.SOCK.STREAM, 0, function (err, fd) {
            if (err) {
                console.log('socket open failed');
                debugger;
                return;
            }
            _this.fd = fd;
            var sockAddr = {
                family: syscall_1.SOCK.STREAM,
                port: port,
                addr: ipAddr,
            };
            var buf = new Uint8Array(marshal.socket.SockAddrInDef.length);
            var view = new DataView(buf.buffer, buf.byteOffset);
            var _;
            _a = marshal.Marshal(view, 0, sockAddr, marshal.socket.SockAddrInDef), _ = _a[0], err = _a[1];
            if (err) {
                console.log('marshal failed');
                debugger;
                return;
            }
            syscall_1.syscall.bind(fd, buf, function (bindErr) {
                if (bindErr) {
                    console.log('bind failed: ' + bindErr);
                    debugger;
                    return;
                }
                _this.bound = true;
                if (_this.listenPending)
                    _this._listen();
            });
            var _a;
        });
        return 0;
    };
    TCP.prototype.listen = function (backlog) {
        this.backlog = backlog;
        if (this.bound)
            this._listen();
        else
            this.listenPending = true;
        return 0;
    };
    TCP.prototype.writeBinaryString = function (req, data) {
        var _this = this;
        syscall_1.syscall.pwrite(this.fd, data, -1, function (err) {
            var status = err < 0 ? -1 : 0;
            err = err >= 0 ? undefined : err;
            req.oncomplete(status, _this, req, err);
        });
    };
    TCP.prototype.writeUtf8String = function (req, data) {
        var _this = this;
        syscall_1.syscall.pwrite(this.fd, data, -1, function (err) {
            var status = err < 0 ? -1 : 0;
            err = err >= 0 ? undefined : err;
            req.oncomplete(status, _this, req, err);
        });
    };
    TCP.prototype.readStart = function () {
        this._read();
    };
    TCP.prototype._read = function () {
        var _this = this;
        if (this.fd < 0)
            return;
        syscall_1.syscall.pread(this.fd, 262144, -1, function (err, dataLen, data) {
            var n = dataLen ? dataLen : uv.UV_EOF;
            if (_this.onread) {
                var b = data ? new Buffer(data) : null;
                _this.onread(n, b);
            }
            if (_this.fd !== -1)
                setTimeout(_this._read.bind(_this), 0);
        });
    };
    TCP.prototype._listen = function () {
        var _this = this;
        syscall_1.syscall.listen(this.fd, this.backlog, function (err) {
            if (err) {
                console.log('listen failed: ' + err);
                debugger;
                return;
            }
            _this._accept();
        });
        return;
    };
    TCP.prototype._accept = function () {
        var _this = this;
        syscall_1.syscall.accept(this.fd, function (err, fd, addr, port) {
            if (err) {
                _this.onconnection(err);
            }
            else {
                var handle = new TCP(fd, true);
                handle.addr = addr;
                handle.port = port;
                _this.onconnection(undefined, handle);
            }
            setTimeout(_this._accept.bind(_this), 0);
        });
    };
    return TCP;
}(stream_wrap_1.StreamWrap));
exports.TCP = TCP;
