'use strict';
var syscall_1 = require('./syscall');
var bindingBuffer = require('./binding/buffer');
var bindingUV = require('./binding/uv');
var bindingFs = require('./binding/fs');
var bindingFsEventWrap = require('./binding/fs_event_wrap');
var bindingConstants = require('./binding/constants');
var bindingContextify = require('./binding/contextify');
var bindingProcessWrap = require('./binding/process_wrap');
var bindingPipeWrap = require('./binding/pipe_wrap');
var bindingTTYWrap = require('./binding/tty_wrap');
var bindingSpawnSync = require('./binding/spawn_sync');
var bindingUtil = require('./binding/util');
var bindingTimerWrap = require('./binding/timer_wrap');
var bindingCaresWrap = require('./binding/cares_wrap');
var bindingTCPWrap = require('./binding/tcp_wrap');
var bindingStreamWrap = require('./binding/stream_wrap');
var bindingUDPWrap = require('./binding/udp_wrap');
var bindingHTTPParser = require('./binding/http_parser');
var _bindings = {
    'buffer': bindingBuffer,
    'uv': bindingUV,
    'fs': bindingFs,
    'http_parser': bindingHTTPParser,
    'fs_event_wrap': bindingFsEventWrap,
    'constants': bindingConstants,
    'contextify': bindingContextify,
    'process_wrap': bindingProcessWrap,
    'pipe_wrap': bindingPipeWrap,
    'tty_wrap': bindingTTYWrap,
    'timer_wrap': bindingTimerWrap,
    'cares_wrap': bindingCaresWrap,
    'tcp_wrap': bindingTCPWrap,
    'udp_wrap': bindingUDPWrap,
    'stream_wrap': bindingStreamWrap,
    'spawn_sync': bindingSpawnSync,
    'util': bindingUtil,
};
var Process = (function () {
    function Process(argv, environ) {
        this.queue = [];
        this.draining = false;
        this.argv = argv;
        this.env = environ;
    }
    Process.prototype.init = function (cb) {
        var _this = this;
        syscall_1.syscall.getcwd(function (cwd) {
            _this.pwd = cwd;
            setTimeout(cb);
        });
    };
    Process.prototype.cwd = function () {
        return this.pwd;
    };
    Process.prototype.exit = function (code) {
        setTimeout(function () { syscall_1.syscall.exit(code); }, 0);
    };
    Process.prototype.binding = function (name) {
        if (!(name in _bindings)) {
            console.log('TODO: unimplemented binding ' + name);
            console.trace('TODO: unimplemented binding ' + name);
            return null;
        }
        return _bindings[name];
    };
    Process.prototype.nextTick = function (fun) {
        var args = [];
        for (var _i = 1; _i < arguments.length; _i++) {
            args[_i - 1] = arguments[_i];
        }
        this.queue.push([fun, args]);
        if (!this.draining) {
            setTimeout(this.drainQueue.bind(this), 0);
        }
    };
    Process.prototype.drainQueue = function () {
        if (this.draining) {
            return;
        }
        this.draining = true;
        var currentQueue;
        var len = this.queue.length;
        while (len) {
            currentQueue = this.queue;
            this.queue = [];
            var i = -1;
            while (++i < len) {
                var _a = currentQueue[i], fn = _a[0], args = _a[1];
                fn.apply(this, args);
            }
            len = this.queue.length;
        }
        this.draining = false;
    };
    return Process;
}());
var process = new Process(undefined, {});
self.process = process;
if (typeof self.setTimeout === 'undefined')
    self.setTimeout = superSadSetTimeout;
var fs = require('./fs');
var buffer = require('./buffer');
self.Buffer = buffer.Buffer;
function superSadSetTimeout(cb, ms) {
    'use strict';
    var args = [];
    for (var _i = 2; _i < arguments.length; _i++) {
        args[_i - 2] = arguments[_i];
    }
    return thread.nextTick(cb.bind.apply(cb, [this].concat(args)));
}
var modPipe2 = {
    pipe2: syscall_1.syscall.pipe2.bind(syscall_1.syscall, 0),
};
var modPriority = {
    get: syscall_1.syscall.getpriority.bind(syscall_1.syscall),
    set: syscall_1.syscall.setpriority.bind(syscall_1.syscall),
    Process: 0,
    Pgrp: 1,
    User: 2,
};
function _require(moduleName) {
    'use strict';
    switch (moduleName) {
        case 'fs':
            return fs;
        case 'child_process':
            return require('./child_process');
        case 'net':
            return require('./net');
        case 'http':
            return require('./http');
        case 'path':
            return require('./path');
        case 'readline':
            return require('./readline');
        case 'util':
            return require('./util');
        case 'node-pipe2':
            return modPipe2;
        case 'node-priority':
            return modPriority;
        default:
            throw new ReferenceError('unknown module ' + moduleName);
    }
}
syscall_1.syscall.addEventListener('init', init.bind(this));
function init(data) {
    'use strict';
    var args = data.args[0];
    var environ = data.args[1];
    var debug = data.args[2];
    if (debug)
        debugger;
    process.argv = args;
    process.env = environ;
    process.stdin = new fs.createReadStream('<stdin>', { fd: 0, autoClose: false });
    process.stdout = new fs.createWriteStream('<stdout>', { fd: 1, autoClose: false });
    process.stderr = new fs.createWriteStream('<stderr>', { fd: 2, autoClose: false });
    process.init(function () {
        fs.readFile(args[1], 'utf-8', function (err, contents) {
            if (err) {
                process.stderr.write('error: ' + err, function () {
                    process.exit(1);
                });
                return;
            }
            contents = contents.replace(/^\#\!.*/, '');
            self.process = process;
            self.require = _require;
            self.eval(contents);
        });
    });
}
