export declare enum AF {
    UNSPEC = 0,
    LOCAL = 1,
    UNIX = 1,
    FILE = 1,
    INET = 2,
    INET6 = 10,
}
export declare enum SOCK {
    STREAM = 1,
    DGRAM = 2,
}
export interface Stat {
    dev: number;
    mode: number;
    nlink: number;
    uid: number;
    gid: number;
    rdev: number;
    blksize: number;
    ino: number;
    size: number;
    blocks: number;
    atime: Date;
    mtime: Date;
    ctime: Date;
    birthtime: Date;
}
export declare enum ErrorCode {
    EPERM = 0,
    ENOENT = 1,
    EIO = 2,
    EBADF = 3,
    EACCES = 4,
    EBUSY = 5,
    EEXIST = 6,
    ENOTDIR = 7,
    EISDIR = 8,
    EINVAL = 9,
    EFBIG = 10,
    ENOSPC = 11,
    EROFS = 12,
    ENOTEMPTY = 13,
    ENOTSUP = 14,
}
export declare class ApiError {
    type: ErrorCode;
    message: string;
    code: string;
    constructor(type: ErrorCode, message?: string);
    toString(): string;
}
export declare class SyscallResponse {
    id: number;
    name: string;
    args: any[];
    private static requiredOnData;
    static From(ev: MessageEvent): SyscallResponse;
    constructor(id: number, name: string, args: any[]);
}
export interface SyscallCallback {
    (...args: any[]): void;
}
export interface SignalHandler {
    (data: SyscallResponse): void;
}
export declare class USyscalls {
    private msgIdSeq;
    private port;
    private outstanding;
    private signalHandlers;
    constructor(port: MessagePort);
    exit(code: number): void;
    fork(heap: ArrayBuffer, cb: SyscallCallback): void;
    kill(pid: number, sig: number, cb: SyscallCallback): void;
    wait4(pid: number, options: number, cb: SyscallCallback): void;
    socket(domain: AF, type: SOCK, protocol: number, cb: SyscallCallback): void;
    getsockname(fd: number, cb: SyscallCallback): void;
    getpeername(fd: number, cb: SyscallCallback): void;
    bind(fd: number, sockInfo: Uint8Array, cb: SyscallCallback): void;
    listen(fd: number, backlog: number, cb: SyscallCallback): void;
    accept(fd: number, cb: SyscallCallback): void;
    connect(fd: number, addr: Uint8Array, cb: SyscallCallback): void;
    getcwd(cb: SyscallCallback): void;
    getpid(cb: SyscallCallback): void;
    getppid(cb: SyscallCallback): void;
    spawn(cwd: string, name: string, args: string[], env: string[], files: number[], cb: SyscallCallback): void;
    pipe2(flags: number, cb: SyscallCallback): void;
    getpriority(which: number, who: number, cb: SyscallCallback): void;
    setpriority(which: number, who: number, prio: number, cb: SyscallCallback): void;
    open(path: string | Uint8Array, flags: number | string, mode: number, cb: SyscallCallback): void;
    unlink(path: string, cb: SyscallCallback): void;
    utimes(path: string, atime: number, mtime: number, cb: SyscallCallback): void;
    futimes(fd: number, atime: number, mtime: number, cb: SyscallCallback): void;
    rmdir(path: string, cb: SyscallCallback): void;
    mkdir(path: string, mode: number, cb: SyscallCallback): void;
    close(fd: number, cb: SyscallCallback): void;
    pwrite(fd: number, buf: string | Uint8Array, pos: number, cb: SyscallCallback): void;
    readdir(path: string, cb: SyscallCallback): void;
    fstat(fd: number, cb: SyscallCallback): void;
    lstat(path: string, cb: SyscallCallback): void;
    chdir(path: string, cb: SyscallCallback): void;
    stat(path: string, cb: SyscallCallback): void;
    ioctl(fd: number, request: number, length: number, cb: SyscallCallback): void;
    readlink(path: string | Uint8Array, cb: SyscallCallback): void;
    getdents(fd: number, length: number, cb: SyscallCallback): void;
    pread(fd: number, length: number, offset: number, cb: SyscallCallback): void;
    addEventListener(type: string, handler: SignalHandler): void;
    private resultHandler(ev);
    private complete(id, args);
    private nextMsgId();
    private post(msgId, name, ...args);
}
export declare function getGlobal(): any;
export declare var syscall: USyscalls;
