'use strict';
var node_binary_marshal_1 = require('node-binary-marshal');
var constants_1 = require('./constants');
var node_fs_stats_1 = require('browserfs-browsix-tmp/dist/node/core/node_fs_stats');
var node_fs_stats_2 = require('browserfs-browsix-tmp/dist/node/core/node_fs_stats');
var SEEK_SET = 0;
var SEEK_CUR = 1;
var SEEK_END = 2;
var S_IFREG = 0x8000;
function assertPath(path) {
    if (typeof path !== 'string') {
        throw new TypeError('Path must be a string. Received ' + path);
    }
}
function normalizeArray(parts, allowAboveRoot) {
    var res = [];
    for (var i = 0; i < parts.length; i++) {
        var p = parts[i];
        if (!p || p === '.')
            continue;
        if (p === '..') {
            if (res.length && res[res.length - 1] !== '..') {
                res.pop();
            }
            else if (allowAboveRoot) {
                res.push('..');
            }
        }
        else {
            res.push(p);
        }
    }
    return res;
}
function resolve() {
    var args = [];
    for (var _i = 0; _i < arguments.length; _i++) {
        args[_i - 0] = arguments[_i];
    }
    var resolvedPath = '';
    var resolvedAbsolute = false;
    for (var i = args.length - 1; i >= -1 && !resolvedAbsolute; i--) {
        var path = (i >= 0) ? args[i] : '/';
        assertPath(path);
        if (path === '')
            continue;
        resolvedPath = path + '/' + resolvedPath;
        resolvedAbsolute = path[0] === '/';
    }
    resolvedPath = normalizeArray(resolvedPath.split('/'), !resolvedAbsolute).join('/');
    return ((resolvedAbsolute ? '/' : '') + resolvedPath) || '.';
}
exports.resolve = resolve;
var RegularFile = (function () {
    function RegularFile(kernel, fd) {
        this.kernel = kernel;
        this.fd = fd;
        this.refCount = 1;
        this.pos = 0;
    }
    RegularFile.prototype.read = function (buf, pos, cb) {
        var _this = this;
        if (pos < 0)
            pos = this.pos;
        this.kernel.fs.read(this.fd, buf, 0, buf.length, pos, function (err, len) {
            if (!err && len)
                _this.pos += len;
            cb(err, len);
        });
    };
    RegularFile.prototype.write = function (buf, pos, cb) {
        var _this = this;
        if (pos < 0)
            pos = this.pos;
        this.kernel.fs.write(this.fd, buf, 0, buf.length, pos, function (err, len) {
            if (!err && len)
                _this.pos += len;
            cb(err, len);
        });
    };
    RegularFile.prototype.stat = function (cb) {
        this.kernel.fs.fstat(this.fd, function (err, stats) {
            if (!err && stats) {
                if (!stats.mode)
                    stats.mode = 438;
                stats.mode |= S_IFREG;
            }
            cb(err, stats);
        });
    };
    RegularFile.prototype.readdir = function (cb) {
        setTimeout(cb, 0, 'cant readdir on normal file');
    };
    RegularFile.prototype.llseek = function (offhi, offlo, whence, cb) {
        var _this = this;
        if (whence === SEEK_CUR) {
            this.pos += offlo;
        }
        else if (whence === SEEK_SET) {
            this.pos = offlo;
        }
        else if (whence === SEEK_END) {
            this.kernel.fs.fstat(this.fd, function (err, stats) {
                if (err || !stats)
                    return cb(err, -1);
                _this.pos = stats.size + offlo;
                cb(0, _this.pos);
            });
            return;
        }
        cb(0, this.pos);
    };
    RegularFile.prototype.ref = function () {
        this.refCount++;
    };
    RegularFile.prototype.unref = function () {
        this.refCount--;
        if (!this.refCount) {
            this.kernel.fs.close(this.fd);
            this.fd = undefined;
        }
    };
    return RegularFile;
}());
exports.RegularFile = RegularFile;
var DirFile = (function () {
    function DirFile(kernel, path) {
        this.kernel = kernel;
        this.path = path;
        this.off = 0;
        this.refCount = 1;
    }
    DirFile.prototype.read = function (buf, pos, cb) {
        setTimeout(cb, 0, 'cant read from a dir -- use readdir');
    };
    DirFile.prototype.write = function (buf, pos, cb) {
        setTimeout(cb, 0, 'cant write to a dir');
    };
    DirFile.prototype.stat = function (cb) {
        this.kernel.fs.stat(this.path, cb);
    };
    DirFile.prototype.readdir = function (cb) {
        this.kernel.fs.readdir(this.path, cb);
    };
    DirFile.prototype.llseek = function (offhi, offlo, whence, cb) {
        console.log('TODO: dir.llseek');
        cb(0, 0);
    };
    DirFile.prototype.getdents = function (buf, cb) {
        var _this = this;
        this.readdir(function (derr, files) {
            if (derr) {
                console.log('readdir: ' + derr);
                cb(-constants_1.EFAULT);
                return;
            }
            files = files.filter(function (s) { return s !== '.deletedFiles.log'; });
            files.sort();
            files = files.slice(_this.off);
            var dents = files.map(function (n) { return new node_binary_marshal_1.fs.Dirent(-1, node_binary_marshal_1.fs.DT.UNKNOWN, n); });
            var view = new DataView(buf.buffer, buf.byteOffset, buf.byteLength);
            var voff = 0;
            for (var i = 0; i < dents.length; i++) {
                var dent = dents[i];
                if (voff + dent.reclen > buf.byteLength)
                    break;
                var _a = node_binary_marshal_1.Marshal(view, voff, dent, node_binary_marshal_1.fs.DirentDef), len = _a[0], err = _a[1];
                if (err) {
                    console.log('dirent marshal failed: ' + err);
                    cb(-constants_1.EFAULT);
                    return;
                }
                voff += len;
                _this.off++;
            }
            cb(voff);
        });
    };
    DirFile.prototype.ref = function () {
        this.refCount++;
    };
    DirFile.prototype.unref = function () {
        this.refCount--;
        if (!this.refCount) {
            this.path = undefined;
        }
    };
    return DirFile;
}());
exports.DirFile = DirFile;
var NullFile = (function () {
    function NullFile() {
        this.refCount = 1;
        this.pos = 0;
    }
    NullFile.prototype.read = function (buf, pos, cb) {
        cb(null, 0);
    };
    NullFile.prototype.write = function (buf, pos, cb) {
        cb(null, buf.length);
    };
    NullFile.prototype.stat = function (cb) {
        cb(null, new node_fs_stats_1.default(node_fs_stats_2.FileType.FILE, 0, 0x309));
    };
    NullFile.prototype.readdir = function (cb) {
        setTimeout(cb, 0, 'cant readdir on /dev/null');
    };
    NullFile.prototype.llseek = function (offhi, offlo, whence, cb) {
        this.pos = 0;
        cb(0, this.pos);
    };
    NullFile.prototype.ref = function () {
        this.refCount++;
    };
    NullFile.prototype.unref = function () {
        this.refCount--;
        if (!this.refCount) {
            this.fd = undefined;
        }
    };
    return NullFile;
}());
exports.NullFile = NullFile;
