/// <reference path="../../typings/index.d.ts" />
import { SocketFile } from './socket';
import { ExitCallback, OutputCallback, SyscallResult, Syscall, ConnectCallback, IKernel, ITask, IFile, Environment } from './types';
export declare enum AF {
    UNSPEC = 0,
    LOCAL = 1,
    UNIX = 1,
    FILE = 1,
    INET = 2,
    INET6 = 10,
}
export declare enum SOCK {
    STREAM = 1,
    DGRAM = 2,
}
export declare class Syscalls {
    kernel: Kernel;
    constructor(kernel: Kernel);
    getcwd(task: ITask): string;
    personality(task: ITask, kind: number, heap: SharedArrayBuffer, off: number, cb: (err: any) => void): void;
    fork(task: ITask, heap: ArrayBuffer, args: any, cb: (pid: number) => void): void;
    execve(task: ITask, path: string, args: string[], env: Environment, cb: (pid: number) => void): void;
    exit(task: ITask, code: number): void;
    kill(task: ITask, pid: number, sig: number, cb: (err: number) => void): void;
    chdir(task: ITask, path: string, cb: (err: number) => void): void;
    wait4(task: ITask, pid: number, options: number, cb: (pid: number, wstatus?: number, rusage?: any) => void): void;
    getpid(task: ITask): number;
    getppid(task: ITask): number;
    getdents(task: ITask, fd: number, buf: Uint8Array, cb: (err: number) => void): void;
    llseek(task: ITask, fd: number, offhi: number, offlo: number, whence: number, cb: (err: number, off: number) => void): void;
    socket(task: ITask, domain: AF, type: SOCK, protocol: number, cb: (err: number, fd?: number) => void): void;
    bind(task: ITask, fd: number, sockAddr: Uint8Array, cb: (err: number) => void): void;
    getsockname(task: ITask, fd: number, buf: Uint8Array, cb: (err: number, len: number) => void): void;
    getpeername(task: ITask, fd: number, buf: Uint8Array, cb: (err: number, len: number) => void): void;
    listen(task: ITask, fd: number, backlog: number, cb: (err: number) => void): void;
    accept(task: ITask, fd: number, buf: Uint8Array, flags: number, cb: (newFD: number) => void): void;
    connect(task: ITask, fd: number, sockAddr: Uint8Array, cb: ConnectCallback): void;
    spawn(task: ITask, cwd: string, path: string, args: string[], env: string[], files: number[], cb: (err: number, pid: number) => void): void;
    pread(task: ITask, fd: number, buf: Buffer, pos: number, cb: (err: any, len?: number) => void): void;
    pwrite(task: ITask, fd: number, buf: Buffer, pos: number, cb: (err: any, len?: number) => void): void;
    pipe2(task: ITask, flags: number, cb: (err: number, fd1: number, fd2: number) => void): void;
    getpriority(task: ITask, which: number, who: number, cb: (err: number, p: number) => void): void;
    setpriority(task: ITask, which: number, who: number, prio: number, cb: (err: number, p: number) => void): void;
    readdir(task: ITask, path: string, cb: (err: any, files: string[]) => void): void;
    rename(task: ITask, relOldName: string, relNewName: string, cb: (err: number) => void): void;
    open(task: ITask, path: string, flags: string, mode: number, cb: (err: number, fd: number) => void): void;
    dup(task: ITask, fd1: number, cb: (ret: number) => void): void;
    dup3(task: ITask, fd1: number, fd2: number, opts: number, cb: (ret: number) => void): void;
    unlink(task: ITask, path: string, cb: (err: any) => void): void;
    utimes(task: ITask, path: string, atimets: number, mtimets: number, cb: (err: any) => void): void;
    futimes(task: ITask, fd: number, atimets: number, mtimets: number, cb: (err: any) => void): void;
    rmdir(task: ITask, path: string, cb: (err: any) => void): void;
    mkdir(task: ITask, path: string, mode: number, cb: (err: any) => void): void;
    close(task: ITask, fd: number, cb: (err: number) => void): void;
    access(task: ITask, path: string, flags: number, cb: (err: number) => void): void;
    fstat(task: ITask, fd: number, buf: Uint8Array, cb: (err: number) => void): void;
    lstat(task: ITask, path: string, buf: Uint8Array, cb: (err: number) => void): void;
    stat(task: ITask, path: string, buf: Uint8Array, cb: (err: number) => any): void;
    readlink(task: ITask, path: string, cb: (err: number, buf?: Uint8Array) => any): void;
    ioctl(task: ITask, fd: number, request: number, length: number, cb: (err: number) => void): void;
}
export declare class Kernel implements IKernel {
    fs: any;
    nCPUs: number;
    runQueues: ITask[][];
    outstanding: number;
    debug: boolean;
    portWaiters: {
        [port: number]: Function;
    };
    syscallsCommon: Syscalls;
    ports: {
        [port: number]: SocketFile;
    };
    private tasks;
    private taskIdSeq;
    private syscalls;
    constructor(fs: any, nCPUs: number, args: BootArgs);
    once(event: string, cb: Function): any;
    system(cmd: string, onExit: ExitCallback, onStdout: OutputCallback, onStderr: OutputCallback): void;
    httpRequest(url: string, cb: any): void;
    wait(pid: number): void;
    exit(task: Task, code: number): void;
    kill(pid: number): void;
    signal(pid: number, sig: number, cb: (err: number) => void): void;
    unbind(s: IFile, addr: string, port: number): any;
    bind(s: SocketFile, addr: string, port: number, cb: (err: number) => void): any;
    connect(f: IFile, addr: string, port: number, cb: ConnectCallback): void;
    doSyscall(syscall: Syscall): void;
    spawn(parent: Task, cwd: string, name: string, args: string[], envArray: string[], filesArray: number[], cb: (err: number, pid: number) => void): void;
    fork(parent: Task, heap: ArrayBuffer, forkArgs: any, cb: (pid: number) => void): void;
    private nextTaskId();
}
export declare enum TaskState {
    Starting = 0,
    Running = 1,
    Interruptable = 2,
    Zombie = 3,
}
export declare class Task implements ITask {
    kernel: IKernel;
    worker: Worker;
    state: TaskState;
    pid: number;
    files: {
        [n: number]: IFile;
    };
    exitCode: number;
    exePath: string;
    exeFd: any;
    blobUrl: string;
    args: string[];
    env: Environment;
    pendingExePath: string;
    pendingArgs: string[];
    pendingEnv: Environment;
    cwd: string;
    waitQueue: any[];
    heapu8: Uint8Array;
    heap32: Int32Array;
    sheap: SharedArrayBuffer;
    waitOff: number;
    heap: ArrayBuffer;
    forkArgs: any;
    parent: Task;
    children: Task[];
    onExit: ExitCallback;
    priority: number;
    private msgIdSeq;
    private onRunnable;
    private syncSyscall;
    private syncSyscallStart;
    private timeWorkerStart;
    private timeFirstMsg;
    private timeSyscallTotal;
    constructor(kernel: Kernel, parent: Task, pid: number, cwd: string, filename: string, args: string[], env: Environment, files: {
        [n: number]: IFile;
    }, blobUrl: string, heap: ArrayBuffer, forkArgs: any, cb: (err: number, pid: number) => void);
    personality(kind: number, sab: SharedArrayBuffer, off: number, cb: (err: number) => void): void;
    exec(filename: string, args: string[], env: Environment, cb: (err: any, pid: number) => void): void;
    chdir(path: string, cb: Function): void;
    allocFD(): number;
    addFile(f: IFile): number;
    fileOpened(err: any, fd: any): void;
    fileRead(fd: number, err: any, bytesRead: number, buf: Buffer): void;
    blobReady(blobUrl: string): void;
    setPriority(prio: number): number;
    wait4(pid: number, options: number, cb: (pid: number, wstatus?: number, rusage?: any) => void): void;
    childDied(pid: number, code: number): void;
    signal(name: string, args: any[], transferrable?: any[]): void;
    schedule(msg: SyscallResult, transferrable?: any[]): void;
    exit(code: number): void;
    private nextMsgId();
    private syscallHandler(ev);
}
export interface BootCallback {
    (err: any, kernel: IKernel): void;
}
export interface BootArgs {
    fsType?: string;
    fsArgs?: any[];
    ttyParent?: Element;
    readOnly?: boolean;
}
export declare function Boot(fsType: string, fsArgs: any[], cb: BootCallback, args?: BootArgs): void;
export declare function BootWith(rootFs: any, cb: BootCallback, args?: BootArgs): void;
export declare function BrowsixFSes(): any;
