'use strict';
var _this = this;
var constants = require('./constants');
var pipe_1 = require('./pipe');
var socket_1 = require('./socket');
var file_1 = require('./file');
var types_1 = require('./types');
var http_parser_1 = require('./http_parser');
var bfs = require('browserfs-browsix-tmp');
var marshal = require('node-binary-marshal');
var buffer_1 = require('../browser-node/binding/buffer');
var DEBUG = false;
var STRACE = false;
var Buffer;
if (typeof Atomics !== 'undefined') {
    if (!Atomics.wait && Atomics.futexWait)
        Atomics.wait = Atomics.futexWait;
    if (!Atomics.wake && Atomics.futexWake)
        Atomics.wake = Atomics.futexWake;
}
function getRandomPort() {
    var min = 2 << 9;
    var max = 2 << 15;
    return Math.floor(Math.random() * (max - min)) + min;
}
if (true) {
    var g_1 = global;
    var timeouts_1 = [];
    var messageName_1 = "zero-timeout-message";
    var canUsePostMessage = function () {
        if (typeof g_1.importScripts !== 'undefined' || !g_1.postMessage)
            return false;
        var isAsync = true;
        var oldOnMessage = g_1.onmessage;
        g_1.onmessage = function () { isAsync = false; };
        g_1.postMessage('', '*');
        g_1.onmessage = oldOnMessage;
        return isAsync;
    };
    if (canUsePostMessage()) {
        g_1.setImmediate = function (fn) {
            var args = [];
            for (var _i = 1; _i < arguments.length; _i++) {
                args[_i - 1] = arguments[_i];
            }
            timeouts_1.push([fn, args]);
            g_1.postMessage(messageName_1, "*");
        };
        var handleMessage = function (event) {
            if (event.source === self && event.data === messageName_1) {
                if (event.stopPropagation)
                    event.stopPropagation();
                else
                    event.cancelBubble = true;
            }
            if (timeouts_1.length > 0) {
                var _a = timeouts_1.shift(), fn = _a[0], args = _a[1];
                return fn.apply(_this, args);
            }
        };
        g_1.addEventListener('message', handleMessage, true);
    }
    else {
        g_1.setImmediate = function (fn) {
            var args = [];
            for (var _i = 1; _i < arguments.length; _i++) {
                args[_i - 1] = arguments[_i];
            }
            return setTimeout.apply(_this, [fn, 0].concat(args));
        };
    }
}
if (typeof window === 'undefined' || typeof window.Worker === 'undefined')
    var Worker = require('webworker-threads').Worker;
else
    var Worker = window.Worker;
var PER_NONBLOCK = 0x40;
var PER_BLOCKING = 0x80;
var ENOTTY = 25;
var ENOENT = constants.ENOENT;
var EACCES = constants.EACCES;
var EINVAL = constants.EINVAL;
var EISDIR = constants.EISDIR;
var F_OK = constants.F_OK;
var R_OK = constants.R_OK;
var W_OK = constants.W_OK;
var X_OK = constants.X_OK;
var S_IRUSR = constants.S_IRUSR;
var S_IWUSR = constants.S_IWUSR;
var S_IXUSR = constants.S_IXUSR;
var O_APPEND = constants.O_APPEND;
var O_CREAT = constants.O_CREAT;
var O_EXCL = constants.O_EXCL;
var O_RDONLY = constants.O_RDONLY;
var O_RDWR = constants.O_RDWR;
var O_SYNC = constants.O_SYNC;
var O_TRUNC = constants.O_TRUNC;
var O_WRONLY = constants.O_WRONLY;
var O_NONBLOCK = constants.O_NONBLOCK;
var O_DIRECTORY = constants.O_DIRECTORY;
var PRIO_MIN = -20;
var PRIO_MAX = 20;
var O_CLOEXEC = 0x80000;
var O_LARGEFILE = 0x8000;
var F_DUPFD = 0;
var F_GETFD = 1;
var F_SETFD = 2;
var F_GETFL = 3;
var F_SETFL = 4;
var F_SETOWN = 8;
var F_GETOWN = 9;
var F_SETSIG = 10;
var F_GETSIG = 11;
var F_GETLK = 12;
var F_SETLK = 13;
var F_SETLKW = 14;
var F_SETOWN_EX = 15;
var F_GETOWN_EX = 16;
var F_GETOWNER_UIDS = 17;
function flagsToString(flag) {
    'use strict';
    if (typeof flag !== 'number') {
        return flag;
    }
    if (flag & O_NONBLOCK) {
        console.log('TODO: nonblocking flag');
    }
    flag &= ~(O_CLOEXEC | O_LARGEFILE | O_DIRECTORY | O_NONBLOCK);
    switch (flag) {
        case O_RDONLY:
            return 'r';
        case O_WRONLY:
            return 'w';
        case O_RDONLY | O_SYNC:
            return 'rs';
        case O_RDWR:
            return 'r+';
        case O_RDWR | O_SYNC:
            return 'rs+';
        case O_CREAT | O_WRONLY:
        case O_CREAT | O_WRONLY | O_TRUNC:
            return 'w';
        case O_CREAT | O_WRONLY | O_EXCL:
        case O_CREAT | O_WRONLY | O_EXCL | O_TRUNC:
            return 'wx';
        case O_CREAT | O_RDWR:
        case O_CREAT | O_RDWR | O_TRUNC:
            return 'w+';
        case O_CREAT | O_RDWR | O_EXCL:
        case O_CREAT | O_RDWR | O_EXCL | O_TRUNC:
            return 'wx+';
        case O_APPEND | O_CREAT | O_WRONLY:
            return 'a';
        case O_APPEND | O_CREAT | O_WRONLY | O_EXCL:
            return 'ax';
        case O_APPEND | O_CREAT | O_RDWR:
            return 'a+';
        case O_APPEND | O_CREAT | O_RDWR | O_EXCL:
            return 'ax+';
    }
    throw new Error('Unknown file open flag: ' + flag);
}
(function (AF) {
    AF[AF["UNSPEC"] = 0] = "UNSPEC";
    AF[AF["LOCAL"] = 1] = "LOCAL";
    AF[AF["UNIX"] = 1] = "UNIX";
    AF[AF["FILE"] = 1] = "FILE";
    AF[AF["INET"] = 2] = "INET";
    AF[AF["INET6"] = 10] = "INET6";
})(exports.AF || (exports.AF = {}));
var AF = exports.AF;
;
(function (SOCK) {
    SOCK[SOCK["STREAM"] = 1] = "STREAM";
    SOCK[SOCK["DGRAM"] = 2] = "DGRAM";
})(exports.SOCK || (exports.SOCK = {}));
var SOCK = exports.SOCK;
function syncSyscalls(sys, task, sysret) {
    var _this = this;
    var dataViewWorks = true;
    try {
        var _ = new DataView(new SharedArrayBuffer(32), 0, 32);
    }
    catch (e) {
        dataViewWorks = false;
    }
    function bufferAt(off, len) {
        if (dataViewWorks) {
            return new Buffer(new DataView(task.sheap, off, len));
        }
        else {
            var tmp = new Uint8Array(task.sheap, off, len);
            var notShared = new ArrayBuffer(len);
            new Uint8Array(notShared).set(tmp);
            return new Buffer(new DataView(notShared));
        }
    }
    function arrayAt(off, len) {
        if (dataViewWorks) {
            return task.heapu8.subarray(off, off + len);
        }
        else {
            var tmp = new Uint8Array(task.sheap, off, len);
            var notShared = new ArrayBuffer(len);
            var notSharedArray = new Uint8Array(notShared);
            notSharedArray.set(tmp);
            return notSharedArray;
        }
    }
    function stringAt(ptr) {
        var s = new Uint8Array(task.sheap, ptr);
        var len = 0;
        while (s[len] !== 0)
            len++;
        return buffer_1.utf8Slice(s, 0, len);
    }
    function stringArrayAt(ptr) {
        if (!dataViewWorks) {
            console.log('FIXME: get data view working');
            return;
        }
        var arr = [];
        var i = 0;
        for (var i_1 = 0; task.heap32[(ptr + i_1) >> 2] !== 0; i_1 += 4) {
            var s = stringAt(task.heap32[(ptr + i_1) >> 2]);
            arr.push(s);
        }
        return arr;
    }
    var table = {
        3: function (fd, bufp, len) {
            var buf = bufferAt(bufp, len);
            sys.pread(task, fd, buf, -1, function (err, len) {
                if (err) {
                    if (typeof err === 'number')
                        len = err;
                    else
                        len = -1;
                }
                sysret(len);
            });
        },
        4: function (fd, bufp, len) {
            var buf = bufferAt(bufp, len);
            sys.pwrite(task, fd, buf, -1, function (err, len) {
                if (err) {
                    if (typeof err === 'number')
                        len = err;
                    else
                        len = -1;
                }
                sysret(len);
            });
        },
        5: function (pathp, flags, mode) {
            var path = stringAt(pathp);
            var sflags = flagsToString(flags);
            sys.open(task, path, sflags, mode, function (err, fd) {
                if ((typeof err === 'number') && err < 0)
                    fd = err;
                sysret(fd | 0);
            });
        },
        6: function (fd) {
            sys.close(task, fd, sysret);
        },
        10: function (pathp) {
            var path = stringAt(pathp);
            sys.unlink(task, path, function (err) {
                if (err && err.errno)
                    sysret(-err.errno);
                else if (err)
                    sysret(-1);
                else
                    sysret(0);
            });
        },
        11: function (filenamep, argv, envp) {
            var filename = stringAt(filenamep);
            var args = stringArrayAt(argv);
            var env = stringArrayAt(envp);
            var senv = {};
            for (var i = 0; i < env.length; i++) {
                var pair = env[i];
                var n = pair.indexOf('=');
                if (n > 0)
                    senv[pair.slice(0, n)] = pair.slice(n + 1);
            }
            sys.execve(task, filename, args, senv, sysret);
        },
        12: function (pathnamep) {
            var pathname = stringAt(pathnamep);
            sys.chdir(task, pathname, sysret);
        },
        20: function (fd, op) {
            sysret(sys.getpid(task));
        },
        33: function (pathp, amode) {
            var path = stringAt(pathp);
            sys.access(task, path, amode, sysret);
        },
        37: function (pid, sig) {
            sys.kill(task, pid, sig, sysret);
        },
        38: function (oldNamep, newNamep) {
            var oldName = stringAt(oldNamep);
            var newName = stringAt(newNamep);
            sys.rename(task, oldName, newName, sysret);
        },
        39: function (pathp, mode) {
            var path = stringAt(pathp);
            sys.mkdir(task, path, mode, sysret);
        },
        40: function (pathp) {
            var path = stringAt(pathp);
            sys.rmdir(task, path, sysret);
        },
        41: function (fd1) {
            sys.dup(task, fd1, sysret);
        },
        42: function (pipefd, flags) {
            sys.pipe2(task, flags, function (err, fd1, fd2) {
                if (!err) {
                    task.heap32[(pipefd >> 2)] = fd1;
                    task.heap32[(pipefd >> 2) + 1] = fd2;
                }
                sysret(err);
            });
        },
        54: function (fd, op) {
            sys.ioctl(task, fd, op, -1, sysret);
        },
        64: function (fd, op) {
            sysret(sys.getppid(task));
        },
        114: function (pid, wstatus, options, rusage) {
            sys.wait4(task, pid, options, function (pid, wstatus, rusage) {
                if (rusage === void 0) { rusage = null; }
                wstatus = wstatus | 0;
                if (wstatus)
                    task.heap32[wstatus >> 2] = wstatus;
                if (rusage)
                    console.log('FIXME: wait4 rusage');
                sysret(pid);
            });
        },
        140: function (fd, offhi, offlo, resultp, whence) {
            sys.llseek(task, fd, offhi, offlo, whence, function (err, off) {
                if (!err) {
                    task.heap32[resultp >> 2] = off;
                }
                sysret(err);
            });
        },
        174: function (act, oldact) {
            sysret(0);
        },
        183: function (bufp, size) {
            var cwd = buffer_1.utf8ToBytes(sys.getcwd(task));
            if (cwd.byteLength > size)
                cwd = cwd.subarray(0, size);
            task.heapu8.subarray(bufp, bufp + size).set(cwd);
            sysret(cwd.byteLength);
        },
        195: function (pathp, bufp) {
            var path = stringAt(pathp);
            var len = marshal.fs.StatDef.length;
            var buf = arrayAt(bufp, len);
            sys.stat(task, path, buf, function () {
                if (!dataViewWorks)
                    task.heapu8.subarray(bufp, bufp + len).set(buf);
                sysret.apply(this, arguments);
            });
        },
        196: function (pathp, bufp) {
            var path = stringAt(pathp);
            var len = marshal.fs.StatDef.length;
            var buf = arrayAt(bufp, len);
            sys.lstat(task, path, buf, function () {
                if (!dataViewWorks)
                    task.heapu8.subarray(bufp, bufp + len).set(buf);
                sysret.apply(this, arguments);
            });
        },
        197: function (fd, bufp) {
            var len = marshal.fs.StatDef.length;
            var buf = arrayAt(bufp, len);
            sys.fstat(task, fd, buf, function () {
                if (!dataViewWorks)
                    task.heapu8.subarray(bufp, bufp + len).set(buf);
                sysret.apply(this, arguments);
            });
        },
        220: function (fd, dirp, count) {
            var buf = arrayAt(dirp, count);
            sys.getdents(task, fd, buf, sysret);
        },
        221: function (fd, cmd, arg) {
            switch (cmd) {
                case F_DUPFD:
                    return sys.dup3(task, fd, arg, 0, sysret);
                case F_GETFD:
                    console.log('TODO: fcntl(GETFD)');
                    break;
                case F_SETFD:
                    console.log('TODO: fcntl(SETFD)');
                    break;
                case F_GETFL:
                    console.log('TODO: fcntl(GETFL)');
                    break;
                case F_SETFL:
                    console.log('TODO: fcntl(SETFL)');
                    break;
                case F_GETLK:
                    console.log('TODO: fcntl(GETLK)');
                    break;
                case F_SETLK:
                    console.log('TODO: fcntl(SETLK)');
                    break;
                case F_SETLKW:
                    console.log('TODO: fcntl(SETLKW)');
                    break;
                case F_GETOWN_EX:
                    console.log('TODO: fcntl(GETOWN_EX)');
                    break;
                case F_SETOWN:
                    console.log('TODO: fcntl(SETOWN)');
                    break;
                case F_GETOWN:
                    console.log('TODO: fcntl(GETOWN)');
                    break;
                default:
                    console.log('TODO: unrecognized fctl64 cmd: ' + cmd);
            }
            sysret(0);
        },
        252: function (code) {
            sys.exit(task, code);
        },
        330: function (fd1, fd2, flags) {
            sys.dup3(task, fd1, fd2, flags, sysret);
        },
    };
    return function (n, args) {
        if (!(n in table)) {
            console.log('sync syscall: unknown ' + n);
            sysret(-constants.ENOTSUP);
            return;
        }
        if (STRACE)
            console.log('[' + task.pid + '] \tsys_' + n + '\t' + args[0]);
        table[n].apply(_this, args);
    };
}
var AsyncSyscalls = (function () {
    function AsyncSyscalls(sys) {
        this.sys = sys;
    }
    AsyncSyscalls.prototype.getcwd = function (ctx) {
        ctx.complete(buffer_1.utf8ToBytes(this.sys.getcwd(ctx.task)));
    };
    AsyncSyscalls.prototype.personality = function (ctx, kind, heap, off) {
        this.sys.personality(ctx.task, kind, heap, off, ctx.complete.bind(ctx));
    };
    AsyncSyscalls.prototype.fork = function (ctx, heap, args) {
        this.sys.fork(ctx.task, heap, args, ctx.complete.bind(ctx));
    };
    AsyncSyscalls.prototype.kill = function (ctx, pid, sig) {
        this.sys.kill(ctx.task, pid, sig, ctx.complete.bind(ctx));
    };
    AsyncSyscalls.prototype.execve = function (ctx, filename, args, env) {
        var file = buffer_1.utf8Slice(filename, 0, filename.length);
        var sargs = [];
        var senv = {};
        for (var i = 0; i < args.length; i++) {
            var arg = args[i];
            sargs[i] = buffer_1.utf8Slice(arg, 0, arg.length);
        }
        for (var i = 0; i < env.length; i++) {
            var pair = buffer_1.utf8Slice(env[i], 0, env[i].length);
            var n = pair.indexOf('=');
            if (n > 0)
                senv[pair.slice(0, n)] = pair.slice(n + 1);
        }
        this.sys.execve(ctx.task, file, sargs, senv, ctx.complete.bind(ctx));
    };
    AsyncSyscalls.prototype.fcntl64 = function (ctx, fd, cmd, arg) {
        switch (cmd) {
            case F_DUPFD:
                return this.sys.dup3(ctx.task, fd, arg, 0, ctx.complete.bind(ctx));
            case F_GETFD:
                console.log('TODO: a-fcntl(GETFD)');
                break;
            case F_SETFD:
                console.log('TODO: a-fcntl(SETFD)');
                break;
            case F_GETFL:
                console.log('TODO: a-fcntl(GETFL)');
                break;
            case F_SETFL:
                console.log('TODO: a-fcntl(SETFL)');
                break;
            case F_GETLK:
                console.log('TODO: a-fcntl(GETLK)');
                break;
            case F_SETLK:
                console.log('TODO: a-fcntl(SETLK)');
                break;
            case F_SETLKW:
                console.log('TODO: a-fcntl(SETLKW)');
                break;
            case F_GETOWN_EX:
                console.log('TODO: a-fcntl(GETOWN_EX)');
                break;
            case F_SETOWN:
                console.log('TODO: a-fcntl(SETOWN)');
                break;
            case F_GETOWN:
                console.log('TODO: a-fcntl(GETOWN)');
                break;
            default:
                console.log('TODO: unrecognized fctl64 cmd: ' + cmd);
        }
        ctx.complete(0);
    };
    AsyncSyscalls.prototype.exit = function (ctx, code) {
        if (!code)
            code = 0;
        this.sys.exit(ctx.task, code);
    };
    AsyncSyscalls.prototype.chdir = function (ctx, p) {
        var s;
        if (p instanceof Uint8Array)
            s = buffer_1.utf8Slice(p, 0, p.length);
        else
            s = p;
        this.sys.chdir(ctx.task, s, ctx.complete.bind(ctx));
    };
    AsyncSyscalls.prototype.wait4 = function (ctx, pid, options) {
        this.sys.wait4(ctx.task, pid, options, function (pid, wstatus, rusage) {
            if (rusage === void 0) { rusage = null; }
            wstatus = wstatus | 0;
            ctx.complete(pid, wstatus, rusage);
        });
    };
    AsyncSyscalls.prototype.getpid = function (ctx) {
        ctx.complete(0, this.sys.getpid(ctx.task));
    };
    AsyncSyscalls.prototype.getppid = function (ctx) {
        ctx.complete(0, this.sys.getppid(ctx.task));
    };
    AsyncSyscalls.prototype.getdents = function (ctx, fd, length) {
        var buf = new Uint8Array(length);
        this.sys.getdents(ctx.task, fd, buf, function (err) {
            if (err <= 0)
                buf = null;
            ctx.complete(err, buf);
        });
    };
    AsyncSyscalls.prototype.llseek = function (ctx, fd, offhi, offlo, whence) {
        this.sys.llseek(ctx.task, fd, offhi, offlo, whence, ctx.complete.bind(ctx));
    };
    AsyncSyscalls.prototype.socket = function (ctx, domain, type, protocol) {
        this.sys.socket(ctx.task, domain, type, protocol, ctx.complete.bind(ctx));
    };
    AsyncSyscalls.prototype.bind = function (ctx, fd, sockAddr) {
        this.sys.bind(ctx.task, fd, sockAddr, ctx.complete.bind(ctx));
    };
    AsyncSyscalls.prototype.getsockname = function (ctx, fd) {
        var buf = new Uint8Array(marshal.socket.SockAddrInDef.length);
        this.sys.getsockname(ctx.task, fd, buf, function (err, len) {
            ctx.complete(err, buf);
        });
    };
    AsyncSyscalls.prototype.getpeername = function (ctx, fd) {
        var buf = new Uint8Array(marshal.socket.SockAddrInDef.length);
        this.sys.getpeername(ctx.task, fd, buf, function (err, len) {
            ctx.complete(err, buf);
        });
    };
    AsyncSyscalls.prototype.listen = function (ctx, fd, backlog) {
        this.sys.listen(ctx.task, fd, backlog, ctx.complete.bind(ctx));
    };
    AsyncSyscalls.prototype.accept = function (ctx, fd, flags) {
        flags = flags | 0;
        var buf = new Uint8Array(marshal.socket.SockAddrInDef.length);
        this.sys.accept(ctx.task, fd, buf, flags, function (newFD) {
            if (newFD < 0)
                ctx.complete(newFD);
            else
                ctx.complete(undefined, newFD, buf);
        });
    };
    AsyncSyscalls.prototype.connect = function (ctx, fd, sockAddr) {
        this.sys.connect(ctx.task, fd, sockAddr, ctx.complete.bind(ctx));
    };
    AsyncSyscalls.prototype.spawn = function (ctx, icwd, iname, iargs, ienv, files) {
        function toStr(buf) {
            if (typeof buf === 'string') {
                return buf;
            }
            else if (buf instanceof Uint8Array || buf instanceof Array) {
                var len = buf.length;
                if (len > 0 && buf[len - 1] === 0)
                    len--;
                return buffer_1.utf8Slice(buf, 0, len);
            }
            console.log('unreachable');
            return '';
        }
        var cwd = toStr(icwd);
        var name = toStr(iname);
        var args = iargs.map(function (x) { return toStr(x); });
        var env = ienv.map(function (x) { return toStr(x); });
        this.sys.spawn(ctx.task, cwd, name, args, env, files, ctx.complete.bind(ctx));
    };
    AsyncSyscalls.prototype.pread = function (ctx, fd, len, pos) {
        var abuf = new Uint8Array(len);
        var buf = new Buffer(abuf.buffer);
        this.sys.pread(ctx.task, fd, buf, pos, function (err, lenRead) {
            if (err) {
                if (typeof err !== 'number')
                    err = -constants.EIO;
                return ctx.complete(err);
            }
            ctx.complete(0, lenRead, abuf.subarray(0, lenRead));
        });
    };
    AsyncSyscalls.prototype.pwrite = function (ctx, fd, buf, pos) {
        var bbuf = null;
        if (typeof (buf) === 'string') {
            var ubuf = buffer_1.utf8ToBytes(buf);
            bbuf = new Buffer(ubuf);
        }
        else if (!(buf instanceof Buffer) && (buf instanceof Uint8Array)) {
            var ubuf = buf;
            var abuf = ubuf.buffer.slice(ubuf.byteOffset, ubuf.byteOffset + ubuf.byteLength);
            bbuf = new Buffer(abuf);
        }
        else {
            bbuf = buf;
        }
        this.sys.pwrite(ctx.task, fd, bbuf, pos, function (err, len) {
            if (err) {
                if (typeof err !== 'number')
                    err = -constants.EIO;
                return ctx.complete(err);
            }
            ctx.complete(0, len);
        });
    };
    AsyncSyscalls.prototype.pipe2 = function (ctx, flags) {
        this.sys.pipe2(ctx.task, flags, ctx.complete.bind(ctx));
    };
    AsyncSyscalls.prototype.getpriority = function (ctx, which, who) {
        this.sys.getpriority(ctx.task, which, who, ctx.complete.bind(ctx));
    };
    AsyncSyscalls.prototype.setpriority = function (ctx, which, who, prio) {
        this.sys.setpriority(ctx.task, which, who, prio, ctx.complete.bind(ctx));
    };
    AsyncSyscalls.prototype.readdir = function (ctx, path) {
        var spath;
        if (path instanceof Uint8Array)
            spath = buffer_1.utf8Slice(path, 0, path.length);
        else
            spath = path;
        this.sys.readdir(ctx.task, spath, ctx.complete.bind(ctx));
    };
    AsyncSyscalls.prototype.rename = function (ctx, oldName, newName) {
        var sOldName;
        if (oldName instanceof Uint8Array)
            sOldName = buffer_1.utf8Slice(oldName, 0, oldName.length);
        else
            sOldName = oldName;
        var sNewName;
        if (newName instanceof Uint8Array)
            sNewName = buffer_1.utf8Slice(newName, 0, newName.length);
        else
            sNewName = newName;
        this.sys.rename(ctx.task, sOldName, sNewName, ctx.complete.bind(ctx));
    };
    AsyncSyscalls.prototype.open = function (ctx, path, flags, mode) {
        var sflags = flagsToString(flags);
        var spath;
        if (path instanceof Uint8Array)
            spath = buffer_1.utf8Slice(path, 0, path.length);
        else
            spath = path;
        this.sys.open(ctx.task, spath, sflags, mode, ctx.complete.bind(ctx));
    };
    AsyncSyscalls.prototype.dup = function (ctx, fd1) {
        this.sys.dup(ctx.task, fd1, ctx.complete.bind(ctx));
    };
    AsyncSyscalls.prototype.dup3 = function (ctx, fd1, fd2, opts) {
        this.sys.dup3(ctx.task, fd1, fd2, opts, ctx.complete.bind(ctx));
    };
    AsyncSyscalls.prototype.unlink = function (ctx, path) {
        var spath;
        if (path instanceof Uint8Array)
            spath = buffer_1.utf8Slice(path, 0, path.length);
        else
            spath = path;
        this.sys.unlink(ctx.task, spath, function (err) {
            if (err && err.errno)
                ctx.complete(-err.errno);
            else if (err)
                ctx.complete(-1);
            else
                ctx.complete(0);
        });
    };
    AsyncSyscalls.prototype.utimes = function (ctx, path, atimets, mtimets) {
        var spath;
        if (path instanceof Uint8Array)
            spath = buffer_1.utf8Slice(path, 0, path.length);
        else
            spath = path;
        this.sys.utimes(ctx.task, spath, atimets, mtimets, ctx.complete.bind(ctx));
    };
    AsyncSyscalls.prototype.futimes = function (ctx, fd, atimets, mtimets) {
        this.sys.futimes(ctx.task, fd, atimets, mtimets, ctx.complete.bind(ctx));
    };
    AsyncSyscalls.prototype.rmdir = function (ctx, path) {
        var spath;
        if (path instanceof Uint8Array)
            spath = buffer_1.utf8Slice(path, 0, path.length);
        else
            spath = path;
        this.sys.rmdir(ctx.task, spath, ctx.complete.bind(ctx));
    };
    AsyncSyscalls.prototype.mkdir = function (ctx, path, mode) {
        var spath;
        if (path instanceof Uint8Array)
            spath = buffer_1.utf8Slice(path, 0, path.length);
        else
            spath = path;
        this.sys.mkdir(ctx.task, spath, mode, ctx.complete.bind(ctx));
    };
    AsyncSyscalls.prototype.close = function (ctx, fd) {
        this.sys.close(ctx.task, fd, ctx.complete.bind(ctx));
    };
    AsyncSyscalls.prototype.access = function (ctx, path, flags) {
        var spath;
        if (path instanceof Uint8Array)
            spath = buffer_1.utf8Slice(path, 0, path.length);
        else
            spath = path;
        this.sys.access(ctx.task, spath, flags, ctx.complete.bind(ctx));
    };
    AsyncSyscalls.prototype.fstat = function (ctx, fd) {
        var buf = new Uint8Array(marshal.fs.StatDef.length);
        this.sys.fstat(ctx.task, fd, buf, function (err) {
            if (err)
                ctx.complete(err, null);
            else
                ctx.complete(0, buf);
        });
    };
    AsyncSyscalls.prototype.lstat = function (ctx, path) {
        var buf = new Uint8Array(marshal.fs.StatDef.length);
        var spath;
        if (path instanceof Uint8Array)
            spath = buffer_1.utf8Slice(path, 0, path.length);
        else
            spath = path;
        this.sys.lstat(ctx.task, spath, buf, function (err) {
            if (err)
                ctx.complete(err);
            else
                ctx.complete(0, buf);
        });
    };
    AsyncSyscalls.prototype.stat = function (ctx, path) {
        var buf = new Uint8Array(marshal.fs.StatDef.length);
        var spath;
        if (path instanceof Uint8Array)
            spath = buffer_1.utf8Slice(path, 0, path.length);
        else
            spath = path;
        this.sys.stat(ctx.task, spath, buf, function (err) {
            if (err)
                ctx.complete(err);
            else
                ctx.complete(0, buf);
        });
    };
    AsyncSyscalls.prototype.readlink = function (ctx, path) {
        var spath;
        if (path instanceof Uint8Array)
            spath = buffer_1.utf8Slice(path, 0, path.length);
        else
            spath = path;
        this.sys.readlink(ctx.task, spath, ctx.complete.bind(ctx));
    };
    AsyncSyscalls.prototype.ioctl = function (ctx, fd, request, length) {
        this.sys.ioctl(ctx.task, fd, request, length, ctx.complete.bind(ctx));
    };
    return AsyncSyscalls;
}());
var Syscalls = (function () {
    function Syscalls(kernel) {
        this.kernel = kernel;
    }
    Syscalls.prototype.getcwd = function (task) {
        return task.cwd;
    };
    Syscalls.prototype.personality = function (task, kind, heap, off, cb) {
        task.personality(kind, heap, off, cb);
    };
    ;
    Syscalls.prototype.fork = function (task, heap, args, cb) {
        this.kernel.fork(task, heap, args, cb);
    };
    Syscalls.prototype.execve = function (task, path, args, env, cb) {
        var fullpath = file_1.resolve(task.cwd, path);
        if (!env['PATH'])
            env['PATH'] = '/usr/bin';
        task.exec(fullpath, args, env, function (err, pid) {
            var nerr = -EACCES;
            if (err && err.errno)
                nerr = -err.errno;
            if (!pid)
                cb(nerr);
        });
    };
    Syscalls.prototype.exit = function (task, code) {
        code = code | 0;
        this.kernel.exit(task, code);
    };
    Syscalls.prototype.kill = function (task, pid, sig, cb) {
        if (sig === constants.SIGKILL || sig === constants.SIGTERM) {
            this.kernel.kill(pid);
            cb(0);
        }
        else {
            this.kernel.signal(pid, sig, cb);
        }
    };
    Syscalls.prototype.chdir = function (task, path, cb) {
        task.chdir(path, cb);
    };
    Syscalls.prototype.wait4 = function (task, pid, options, cb) {
        task.wait4(pid, options, cb);
    };
    Syscalls.prototype.getpid = function (task) {
        return task.pid;
    };
    Syscalls.prototype.getppid = function (task) {
        return task.parent ? task.parent.pid : 0;
    };
    Syscalls.prototype.getdents = function (task, fd, buf, cb) {
        var file = task.files[fd];
        if (!file) {
            cb(-constants.EBADF);
            return;
        }
        if (!(file instanceof file_1.DirFile)) {
            cb(-constants.ENOTDIR);
            return;
        }
        var dir = file;
        dir.getdents(buf, cb);
    };
    Syscalls.prototype.llseek = function (task, fd, offhi, offlo, whence, cb) {
        var file = task.files[fd];
        if (!file) {
            cb(-constants.EBADF, undefined);
            return;
        }
        file.llseek(offhi, offlo, whence, cb);
    };
    Syscalls.prototype.socket = function (task, domain, type, protocol, cb) {
        if (domain === AF.UNSPEC)
            domain = AF.INET;
        if (domain !== AF.INET && type !== SOCK.STREAM) {
            cb(-constants.EAFNOSUPPORT);
            return;
        }
        var f = new socket_1.SocketFile(task);
        var fd = task.addFile(f);
        cb(0, fd);
    };
    Syscalls.prototype.bind = function (task, fd, sockAddr, cb) {
        var info = {};
        var view = new DataView(sockAddr.buffer, sockAddr.byteOffset, sockAddr.byteLength);
        var _a = marshal.Unmarshal(info, view, 0, marshal.socket.SockAddrInDef), _ = _a[0], err = _a[1];
        var addr = info.addr;
        var port = info.port;
        if (port === 0) {
            console.log('port was zero -- changing to 8080');
            port = 8080;
        }
        var file = task.files[fd];
        if (!file) {
            cb(-constants.EBADF);
            return;
        }
        if (socket_1.isSocket(file)) {
            this.kernel.bind(file, addr, port, cb);
            return;
        }
        return cb(-constants.ENOTSOCK);
    };
    Syscalls.prototype.getsockname = function (task, fd, buf, cb) {
        var file = task.files[fd];
        if (!file) {
            cb(-constants.EBADF, -1);
            return;
        }
        if (socket_1.isSocket(file)) {
            var remote = { family: SOCK.STREAM, port: file.port, addr: file.addr };
            var view = new DataView(buf.buffer, buf.byteOffset, buf.byteLength);
            marshal.Marshal(view, 0, remote, marshal.socket.SockAddrInDef);
            cb(0, marshal.socket.SockAddrInDef.length);
            return;
        }
        return cb(-constants.ENOTSOCK, -1);
    };
    Syscalls.prototype.getpeername = function (task, fd, buf, cb) {
        var file = task.files[fd];
        if (!file) {
            cb(-constants.EBADF, -1);
            return;
        }
        if (socket_1.isSocket(file)) {
            if (!file.peer) {
                cb(-constants.ENOTCONN, -1);
            }
            var remote = { family: SOCK.STREAM, port: file.peer.port, addr: file.peer.addr };
            var view = new DataView(buf.buffer, buf.byteOffset, buf.byteLength);
            marshal.Marshal(view, 0, remote, marshal.socket.SockAddrInDef);
            cb(0, marshal.socket.SockAddrInDef.length);
            return;
        }
        return cb(-constants.ENOTSOCK, -1);
    };
    Syscalls.prototype.listen = function (task, fd, backlog, cb) {
        var _this = this;
        var file = task.files[fd];
        if (!file) {
            cb(-constants.EBADF);
            return;
        }
        if (socket_1.isSocket(file)) {
            file.listen(function (err) {
                cb(err | 0);
                if (!err) {
                    var cb_1 = _this.kernel.portWaiters[file.port];
                    if (cb_1) {
                        delete _this.kernel.portWaiters[file.port];
                        cb_1(file.port);
                    }
                }
            });
            return;
        }
        return cb(-constants.ENOTSOCK);
    };
    Syscalls.prototype.accept = function (task, fd, buf, flags, cb) {
        var file = task.files[fd];
        if (!file) {
            cb(-constants.EBADF);
            return;
        }
        if (socket_1.isSocket(file)) {
            file.accept(function (err, s, remoteAddr, remotePort) {
                if (err) {
                    cb(err);
                    return;
                }
                var fd = task.addFile(s);
                if (remoteAddr === 'localhost')
                    remoteAddr = '127.0.0.1';
                var view = new DataView(buf.buffer, buf.byteOffset, buf.byteLength);
                marshal.Marshal(view, 0, { family: 2, port: remotePort, addr: remoteAddr }, marshal.socket.SockAddrInDef);
                cb(fd);
            });
            return;
        }
        return cb(-constants.ENOTSOCK);
    };
    Syscalls.prototype.connect = function (task, fd, sockAddr, cb) {
        var _this = this;
        var info = {};
        var view = new DataView(sockAddr.buffer, sockAddr.byteOffset, sockAddr.byteLength);
        var _a = marshal.Unmarshal(info, view, 0, marshal.socket.SockAddrInDef), _ = _a[0], err = _a[1];
        var addr = info.addr;
        var port = info.port;
        var file = task.files[fd];
        if (!file) {
            cb(-constants.EBADF);
            return;
        }
        if (socket_1.isSocket(file)) {
            file.connect(addr, port, function (err) {
                if (err)
                    cb(err);
                while (true) {
                    var lPort = getRandomPort();
                    if (_this.kernel.ports[lPort])
                        continue;
                    file.port = lPort;
                    file.addr = '127.0.0.1';
                    break;
                }
                cb(0);
            });
            return;
        }
        return cb(-constants.ENOTSOCK);
    };
    Syscalls.prototype.spawn = function (task, cwd, path, args, env, files, cb) {
        var fullpath = file_1.resolve(task.cwd, path);
        this.kernel.spawn(task, cwd, fullpath, args, env, files, cb);
    };
    Syscalls.prototype.pread = function (task, fd, buf, pos, cb) {
        var file = task.files[fd];
        if (!file) {
            cb(-constants.EBADF);
            return;
        }
        if (typeof pos !== 'number')
            pos = -1;
        pos = pos | 0;
        file.read(buf, pos, cb);
    };
    Syscalls.prototype.pwrite = function (task, fd, buf, pos, cb) {
        var file = task.files[fd];
        if (!file) {
            cb(-constants.EBADF);
            return;
        }
        if (typeof pos !== 'number')
            pos = -1;
        pos = pos | 0;
        file.write(buf, pos, cb);
    };
    Syscalls.prototype.pipe2 = function (task, flags, cb) {
        var pipe = new pipe_1.Pipe();
        var n1 = task.addFile(new pipe_1.PipeFile(pipe));
        var n2 = task.addFile(new pipe_1.PipeFile(pipe));
        cb(0, n1, n2);
    };
    Syscalls.prototype.getpriority = function (task, which, who, cb) {
        if (which !== 0 && who !== 0) {
            cb(-constants.EACCES, -1);
            return;
        }
        cb(0, task.priority);
    };
    Syscalls.prototype.setpriority = function (task, which, who, prio, cb) {
        if (which !== 0 && who !== 0) {
            cb(-constants.EACCES, -1);
            return;
        }
        cb(0, task.setPriority(prio));
    };
    Syscalls.prototype.readdir = function (task, path, cb) {
        var fullpath = file_1.resolve(task.cwd, path);
        this.kernel.fs.readdir(fullpath, cb);
    };
    Syscalls.prototype.rename = function (task, relOldName, relNewName, cb) {
        var oldName = file_1.resolve(task.cwd, relOldName);
        var newName = file_1.resolve(task.cwd, relNewName);
        this.kernel.fs.rename(oldName, newName, function (err) {
            if (err && err.errno) {
                cb(-err.errno);
                return;
            }
            else if (err) {
                cb(-1);
                return;
            }
            cb(0);
        });
    };
    Syscalls.prototype.open = function (task, path, flags, mode, cb) {
        var _this = this;
        var fullpath = file_1.resolve(task.cwd, path);
        var f;
        if (fullpath === '/dev/null') {
            f = new file_1.NullFile();
            var n = task.addFile(f);
            cb(0, n);
            return;
        }
        this.kernel.fs.open(fullpath, flags, mode, function (err, fd) {
            if (err && err.errno === EISDIR) {
                f = new file_1.DirFile(_this.kernel, fullpath);
            }
            else if (!err) {
                f = new file_1.RegularFile(_this.kernel, fd);
            }
            else {
                if (typeof err === 'number')
                    cb(err, -1);
                else if (err && err.errno)
                    cb(-err.errno, -1);
                else
                    cb(-1, -1);
                return;
            }
            var n = task.addFile(f);
            cb(0, n);
        });
    };
    Syscalls.prototype.dup = function (task, fd1, cb) {
        var origFile = task.files[fd1];
        if (!origFile) {
            cb(-constants.EBADF);
            return;
        }
        origFile.ref();
        var fd2 = task.allocFD();
        task.files[fd2] = origFile;
        cb(fd2);
    };
    Syscalls.prototype.dup3 = function (task, fd1, fd2, opts, cb) {
        if (fd1 === fd2 || (opts & ~O_CLOEXEC)) {
            cb(-EINVAL);
            return;
        }
        var origFile = task.files[fd1];
        if (!origFile) {
            cb(-constants.EBADF);
            return;
        }
        var oldFile = task.files[fd2];
        if (oldFile) {
            oldFile.unref();
            oldFile = undefined;
        }
        origFile.ref();
        task.files[fd2] = origFile;
        cb(fd2);
    };
    Syscalls.prototype.unlink = function (task, path, cb) {
        var fullpath = file_1.resolve(task.cwd, path);
        this.kernel.fs.unlink(fullpath, cb);
    };
    Syscalls.prototype.utimes = function (task, path, atimets, mtimets, cb) {
        var fullpath = file_1.resolve(task.cwd, path);
        var atime = new Date(atimets * 1000);
        var mtime = new Date(mtimets * 1000);
        this.kernel.fs.utimes(fullpath, atime, mtime, cb);
    };
    Syscalls.prototype.futimes = function (task, fd, atimets, mtimets, cb) {
        var file = task.files[fd];
        if (!file) {
            cb(-constants.EBADF);
            return;
        }
        if (file instanceof pipe_1.Pipe) {
            console.log('TODO futimes: on pipe?');
            cb(-constants.ENOSYS);
            return;
        }
        var atime = new Date(atimets * 1000);
        var mtime = new Date(mtimets * 1000);
        this.kernel.fs.futimes(file, atime, mtime, cb);
    };
    Syscalls.prototype.rmdir = function (task, path, cb) {
        var fullpath = file_1.resolve(task.cwd, path);
        this.kernel.fs.rmdir(fullpath, cb);
    };
    Syscalls.prototype.mkdir = function (task, path, mode, cb) {
        var fullpath = file_1.resolve(task.cwd, path);
        this.kernel.fs.mkdir(fullpath, mode, cb);
    };
    Syscalls.prototype.close = function (task, fd, cb) {
        var file = task.files[fd];
        if (!file) {
            cb(-constants.EBADF);
            return;
        }
        task.files[fd] = undefined;
        file.unref();
        cb(0);
    };
    Syscalls.prototype.access = function (task, path, flags, cb) {
        var fullpath = file_1.resolve(task.cwd, path);
        this.kernel.fs.stat(fullpath, function (err, stats) {
            if (err) {
                cb(-ENOENT);
                return;
            }
            if (flags === F_OK) {
                cb(F_OK);
                return;
            }
            var result = 0;
            if ((flags & R_OK) && !(stats.mode & S_IRUSR)) {
                result = -EACCES;
            }
            if ((flags & W_OK) && !(stats.mode & S_IWUSR)) {
                result = -EACCES;
            }
            if ((flags & X_OK) && !(stats.mode & S_IXUSR)) {
                result = -EACCES;
            }
            cb(result);
        });
    };
    Syscalls.prototype.fstat = function (task, fd, buf, cb) {
        var file = task.files[fd];
        if (!file) {
            cb(-constants.EBADF);
            return;
        }
        file.stat(function (err, stats) {
            if (err && err.errno) {
                cb(-err.errno);
                return;
            }
            else if (err) {
                cb(-1);
                return;
            }
            var view = new DataView(buf.buffer, buf.byteOffset, buf.byteLength);
            marshal.Marshal(view, 0, stats, marshal.fs.StatDef);
            cb(0);
        });
    };
    Syscalls.prototype.lstat = function (task, path, buf, cb) {
        var fullpath = file_1.resolve(task.cwd, path);
        this.kernel.fs.lstat(fullpath, function (err, stats) {
            if (err && err.errno) {
                cb(-err.errno);
                return;
            }
            else if (err) {
                cb(-1);
                return;
            }
            var view = new DataView(buf.buffer, buf.byteOffset, buf.byteLength);
            marshal.Marshal(view, 0, stats, marshal.fs.StatDef);
            cb(0);
        });
    };
    Syscalls.prototype.stat = function (task, path, buf, cb) {
        var fullpath = file_1.resolve(task.cwd, path);
        this.kernel.fs.stat(fullpath, function (err, stats) {
            if (err && err.errno) {
                cb(-err.errno);
                return;
            }
            else if (err) {
                cb(-1);
                return;
            }
            var view = new DataView(buf.buffer, buf.byteOffset, buf.byteLength);
            marshal.Marshal(view, 0, stats, marshal.fs.StatDef);
            cb(0);
        });
    };
    Syscalls.prototype.readlink = function (task, path, cb) {
        var fullpath = file_1.resolve(task.cwd, path);
        this.kernel.fs.readlink(fullpath, function (err, linkString) {
            if (err && err.errno)
                cb(-err.errno);
            else if (err)
                cb(-1);
            else
                cb(0, buffer_1.utf8ToBytes(linkString));
        });
    };
    Syscalls.prototype.ioctl = function (task, fd, request, length, cb) {
        cb(-ENOTTY);
    };
    return Syscalls;
}());
exports.Syscalls = Syscalls;
var Kernel = (function () {
    function Kernel(fs, nCPUs, args) {
        this.debug = DEBUG;
        this.portWaiters = {};
        this.ports = {};
        this.tasks = {};
        this.taskIdSeq = 0;
        this.outstanding = 0;
        this.nCPUs = nCPUs;
        this.fs = fs;
        this.syscallsCommon = new Syscalls(this);
        this.syscalls = new AsyncSyscalls(this.syscallsCommon);
        this.runQueues = [];
        for (var i = PRIO_MIN; i < PRIO_MAX; i++) {
            this.runQueues[i - PRIO_MIN] = [];
        }
    }
    Kernel.prototype.once = function (event, cb) {
        var parts = event.split(':');
        if (parts.length !== 2 || parts[0] !== 'port')
            return 'only supported event is currently port';
        var port = parseInt(parts[1], 10);
        if (!(port >= 1 && port < (2 << 14)))
            return 'invalid port: ' + port;
        this.portWaiters[port] = cb;
    };
    Kernel.prototype.system = function (cmd, onExit, onStdout, onStderr) {
        var _this = this;
        var splitParts = cmd.split(' ');
        var parts;
        if (cmd.match(/[|><&]/) || splitParts[0].match(/[=]/)) {
            parts = ['/bin/sh', '-c', cmd];
        }
        else {
            parts = splitParts.filter(function (s) { return s !== ''; });
        }
        if (parts[0][0] !== '/' && parts[0][0] !== '.')
            parts[0] = '/usr/bin/' + parts[0];
        splitParts = undefined;
        var env = [
            'PWD=/',
            'GOPATH=/',
            'PERL_DESTRUCT_LEVEL=2',
            'USER=browsix',
            'PATH=/usr/bin',
            'LANG=en_US.UTF-8',
            'LC_ALL=en_US.UTF-8',
            'HOME=/',
        ];
        this.spawn(null, '/', parts[0], parts, env, null, function (err, pid) {
            if (err) {
                var code = -666;
                if (err.errno === ENOENT) {
                    code = -constants.ENOENT;
                    onStderr(-1, parts[0] + ": command not found\n");
                }
                onExit(-1, code);
                return;
            }
            var t = _this.tasks[pid];
            t.onExit = onExit;
            var stdout = t.files[1];
            var stderr = t.files[2];
            stdout.addEventListener('write', onStdout);
            stderr.addEventListener('write', onStderr);
        });
    };
    Kernel.prototype.httpRequest = function (url, cb) {
        var port = 80;
        var parts = url.split('://')[1].split('/');
        var host = parts[0];
        var path = '/' + parts.slice(1).join('/');
        if (host.indexOf(':') > -1) {
            var sPort = '';
            _a = host.split(':'), host = _a[0], sPort = _a[1];
            port = parseInt(sPort, 10);
        }
        var req = 'GET ' + url + ' HTTP/1.1\r\n';
        req += 'Host: localhost:' + port + '\r\n';
        req += 'User-Agent: Browsix/1.0\r\n';
        req += 'Accept: */*\r\n\r\n';
        var resp = [];
        var f = new socket_1.SocketFile(null);
        var p = new http_parser_1.HTTPParser(http_parser_1.HTTPParser.RESPONSE);
        var getHeader = function (name) {
            var lname = name.toLowerCase();
            for (var i = 0; i + 1 < p.info.headers.length; i += 2) {
                if (p.info.headers[i].toLowerCase() === lname)
                    return p.info.headers[i + 1];
            }
            return '';
        };
        p.isUserCall = true;
        p[http_parser_1.HTTPParser.kOnHeadersComplete] = function (info) {
        };
        p[http_parser_1.HTTPParser.kOnBody] = function (chunk, off, len) {
            resp.push(chunk.slice(off, off + len));
        };
        p[http_parser_1.HTTPParser.kOnMessageComplete] = function () {
            f.unref();
            var mime = getHeader('Content-Type');
            if (!mime) {
                console.log('WARN: no content-type header');
                mime = 'text/plain';
            }
            var response = Buffer.concat(resp);
            var data = new Uint8Array(response.data.buff.buffer, 0, response.length);
            var blob = new Blob([data], { type: mime });
            var ctx = {
                status: p.info.statusCode,
                response: blob,
            };
            cb.apply(ctx, []);
        };
        var buf = new Buffer(64 * 1024);
        function onRead(err, len) {
            if (err) {
                console.log('http read error: ' + err);
                return;
            }
            p.execute(buf.slice(0, len));
            if (len > 0) {
                buf = new Buffer(64 * 1024);
                f.read(buf, -1, onRead);
            }
        }
        this.connect(f, host, port, function (err) {
            if (err) {
                console.log('connect failed: ' + err);
                return;
            }
            f.read(buf, -1, onRead);
            f.write(new Buffer(req, 'utf8'), -1, function (ierr, len) {
                if (ierr)
                    console.log('err: ' + ierr);
            });
        });
        var _a;
    };
    Kernel.prototype.wait = function (pid) {
        if ((pid in this.tasks) && this.tasks[pid].state === TaskState.Zombie)
            delete this.tasks[pid];
        else
            console.log('wait called for bad pid ' + pid);
    };
    Kernel.prototype.exit = function (task, code) {
        if (task.state === TaskState.Zombie) {
            console.log('warning, got more than 1 exit call from ' + task.pid);
            return;
        }
        task.exit(code);
    };
    Kernel.prototype.kill = function (pid) {
        if (!(pid in this.tasks))
            return;
        var task = this.tasks[pid];
        this.exit(task, -666);
    };
    Kernel.prototype.signal = function (pid, sig, cb) {
        if (pid === -1)
            return cb(-constants.EPERM);
        if (!(pid in this.tasks))
            return cb(-constants.ESRCH);
        this.tasks[pid].signal('signal' + sig, []);
        cb(0);
    };
    Kernel.prototype.unbind = function (s, addr, port) {
        if (!(port in this.ports))
            return;
        if (s !== this.ports[port]) {
            console.log('unbind for wrong port?');
            return;
        }
        delete this.ports[port];
    };
    Kernel.prototype.bind = function (s, addr, port, cb) {
        if (port in this.ports)
            return 'port ' + port + ' already bound';
        this.ports[port] = s;
        s.port = port;
        s.addr = addr;
        cb(0);
    };
    Kernel.prototype.connect = function (f, addr, port, cb) {
        if (addr === '0.0.0.0')
            addr = '127.0.0.1';
        if (addr !== 'localhost' && addr !== '127.0.0.1') {
            console.log('TODO connect(): only localhost supported for now');
            cb(-constants.ECONNREFUSED);
            return;
        }
        if (!(port in this.ports)) {
            cb(-constants.ECONNREFUSED);
            return;
        }
        var listener = this.ports[port];
        if (!listener.isListening) {
            cb(-constants.ECONNREFUSED);
            return;
        }
        var local = f;
        listener.doAccept(local, addr, port, cb);
    };
    Kernel.prototype.doSyscall = function (syscall) {
        if (syscall.name in this.syscalls) {
            if (STRACE) {
                var argfmt = function (arg) {
                    if (arg instanceof Uint8Array) {
                        var len = arg.length;
                        if (len > 0 && arg[len - 1] === 0)
                            len--;
                        return buffer_1.utf8Slice(arg, 0, len);
                    }
                    else {
                        return arg;
                    }
                };
                if (syscall.args === undefined)
                    syscall.args = [undefined];
                var arg = argfmt(syscall.args[0]);
                if (syscall.args[1])
                    arg += '\t' + argfmt(syscall.args[1]);
                console.log('[' + syscall.ctx.task.pid + '|' + syscall.ctx.id + '] \tsys_' + syscall.name + '\t' + arg);
            }
            this.syscalls[syscall.name].apply(this.syscalls, syscall.callArgs());
        }
        else {
            console.log('unknown syscall ' + syscall.name);
        }
    };
    Kernel.prototype.spawn = function (parent, cwd, name, args, envArray, filesArray, cb) {
        var pid = this.nextTaskId();
        envArray = envArray || [];
        var env = {};
        for (var i = 0; i < envArray.length; i++) {
            var s = envArray[i];
            var eq = s.search('=');
            if (eq < 0)
                continue;
            var k = s.substring(0, eq);
            var v = s.substring(eq + 1);
            env[k] = v;
        }
        var files = [];
        if (filesArray && parent) {
            for (var i = 0; i < filesArray.length; i++) {
                var fd = filesArray[i];
                if (!(fd in parent.files)) {
                    console.log('spawn: tried to use bad fd ' + fd);
                    break;
                }
                files[i] = parent.files[fd];
                files[i].ref();
            }
        }
        else {
            files[0] = new file_1.NullFile();
            files[1] = new pipe_1.PipeFile();
            files[2] = new pipe_1.PipeFile();
        }
        var task = new Task(this, parent, pid, '/', name, args, env, files, null, null, null, cb);
        this.tasks[pid] = task;
    };
    Kernel.prototype.fork = function (parent, heap, forkArgs, cb) {
        var pid = this.nextTaskId();
        var cwd = parent.cwd;
        var filename = parent.exePath;
        var args = parent.args;
        var env = parent.env;
        var files = _clone(parent.files);
        for (var i in files) {
            if (!files.hasOwnProperty(i))
                continue;
            if (files[i])
                files[i].ref();
        }
        var blobUrl = parent.blobUrl;
        var forkedTask = new Task(this, parent, pid, cwd, filename, args, env, files, blobUrl, heap, forkArgs, function (err, pid) {
            if (err) {
                console.log('fork failed in kernel: ' + err);
                cb(-1);
            }
            cb(pid);
        });
        this.tasks[pid] = forkedTask;
    };
    Kernel.prototype.nextTaskId = function () {
        return ++this.taskIdSeq;
    };
    return Kernel;
}());
exports.Kernel = Kernel;
(function (TaskState) {
    TaskState[TaskState["Starting"] = 0] = "Starting";
    TaskState[TaskState["Running"] = 1] = "Running";
    TaskState[TaskState["Interruptable"] = 2] = "Interruptable";
    TaskState[TaskState["Zombie"] = 3] = "Zombie";
})(exports.TaskState || (exports.TaskState = {}));
var TaskState = exports.TaskState;
function _clone(obj) {
    if (null === obj || 'object' !== typeof obj)
        return obj;
    var copy = obj.constructor();
    for (var attr in obj) {
        if (obj.hasOwnProperty(attr))
            copy[attr] = obj[attr];
    }
    return copy;
}
var Task = (function () {
    function Task(kernel, parent, pid, cwd, filename, args, env, files, blobUrl, heap, forkArgs, cb) {
        this.files = {};
        this.waitQueue = [];
        this.children = [];
        this.msgIdSeq = 1;
        this.syncSyscallStart = 0.0;
        this.timeWorkerStart = 0.0;
        this.timeFirstMsg = 0.0;
        this.timeSyscallTotal = 0.0;
        this.state = TaskState.Starting;
        this.pid = pid;
        this.parent = parent;
        this.kernel = kernel;
        this.exeFd = null;
        this.cwd = cwd;
        this.priority = 0;
        if (parent) {
            this.priority = parent.priority;
            this.parent.children.push(this);
        }
        this.files = files;
        this.blobUrl = blobUrl;
        this.heap = heap;
        this.forkArgs = forkArgs;
        this.onRunnable = cb;
        if (blobUrl) {
            this.pendingExePath = filename;
            this.pendingArgs = args;
            this.pendingEnv = env;
            this.blobReady(blobUrl);
        }
        else {
            this.exec(filename, args, env, cb);
        }
    }
    Task.prototype.personality = function (kind, sab, off, cb) {
        var _this = this;
        if (kind !== PER_BLOCKING) {
            cb(-EINVAL);
            return;
        }
        this.timeFirstMsg = performance.now();
        this.sheap = sab;
        this.heapu8 = new Uint8Array(sab);
        this.heap32 = new Int32Array(sab);
        this.waitOff = off;
        this.syncSyscall = syncSyscalls(this.kernel.syscallsCommon, this, function (ret) {
            _this.timeSyscallTotal += performance.now() - _this.syncSyscallStart;
            Atomics.store(_this.heap32, (_this.waitOff >> 2) + 1, ret);
            Atomics.store(_this.heap32, _this.waitOff >> 2, 1);
            Atomics.wake(_this.heap32, _this.waitOff >> 2, 1);
        });
        cb(null);
    };
    Task.prototype.exec = function (filename, args, env, cb) {
        var _this = this;
        this.pendingExePath = filename;
        this.pendingArgs = args;
        this.pendingEnv = env;
        this.onRunnable = cb;
        setImmediate(function () {
            _this.kernel.fs.open(filename, 'r', _this.fileOpened.bind(_this));
        });
    };
    Task.prototype.chdir = function (path, cb) {
        var _this = this;
        if (!path.length) {
            cb(-constants.ENOENT);
        }
        if (path[0] !== '/')
            path = file_1.resolve(this.cwd, path);
        this.kernel.fs.stat(path, function (err, stats) {
            if (err) {
                cb(-EACCES);
                return;
            }
            if (!stats.isDirectory()) {
                cb(-constants.ENOTDIR);
                return;
            }
            _this.cwd = path;
            cb(0);
        });
    };
    Task.prototype.allocFD = function () {
        var n = 0;
        for (n = 0; this.files[n]; n++) { }
        return n;
    };
    Task.prototype.addFile = function (f) {
        var n = this.allocFD();
        this.files[n] = f;
        return n;
    };
    Task.prototype.fileOpened = function (err, fd) {
        var _this = this;
        if (err) {
            this.onRunnable(err, undefined);
            var code = -1;
            if (err.errno)
                code = -err.errno;
            this.exit(code);
            return;
        }
        this.exeFd = fd;
        this.kernel.fs.fstat(fd, function (serr, stats) {
            if (serr) {
                _this.onRunnable(serr, undefined);
                var code = -1;
                if (serr.errno)
                    code = -serr.errno;
                _this.exit(code);
                return;
            }
            var buf = new Buffer(stats.size);
            _this.kernel.fs.read(fd, buf, 0, stats.size, 0, _this.fileRead.bind(_this, fd));
        });
    };
    Task.prototype.fileRead = function (fd, err, bytesRead, buf) {
        this.kernel.fs.close(fd, function (e) { });
        if (err) {
            this.onRunnable(err, undefined);
            this.exit(-1);
            return;
        }
        function isShebang(buf) {
            return buf.length > 2
                && buf.readUInt8(0) === 0x23
                && buf.readUInt8(1) === 0x21;
        }
        function isWasm(buf) {
            return buf.length > 4
                && buf.readUInt8(0) === 0x00
                && buf.readUInt8(1) === 0x61
                && buf.readUInt8(2) === 0x73
                && buf.readUInt8(3) === 0x6d;
        }
        if (isShebang(buf)) {
            var newlinePos = buf.indexOf('\n');
            if (newlinePos < 0)
                throw new Error('shebang with no newline: ' + buf);
            var shebang = buf.slice(2, newlinePos).toString();
            buf = buf.slice(newlinePos + 1);
            var parts = shebang.match(/\S+/g);
            var cmd = parts[0];
            if (parts.length === 2 && (parts[0] === '/usr/bin/env' || parts[0] === '/bin/env')) {
                cmd = '/usr/bin/' + parts[1];
            }
            this.pendingArgs[0] = this.pendingExePath;
            this.pendingArgs = [cmd].concat(this.pendingArgs);
            this.kernel.fs.open(cmd, 'r', this.fileOpened.bind(this));
            return;
        }
        else if (isWasm(buf)) {
            this.pendingArgs = ['/usr/bin/ld'].concat(this.pendingArgs);
            this.kernel.fs.open('/usr/bin/ld', 'r', this.fileOpened.bind(this));
            return;
        }
        var jsBytes = new Uint8Array(buf.data.buff.buffer);
        var blob = new Blob([jsBytes], { type: 'text/javascript' });
        jsBytes = undefined;
        var blobUrl = window.URL.createObjectURL(blob);
        this.blobUrl = blobUrl;
        this.blobReady(blobUrl);
    };
    Task.prototype.blobReady = function (blobUrl) {
        var _this = this;
        if (this.worker) {
            this.worker.onmessage = undefined;
            this.worker.terminate();
            this.worker = undefined;
        }
        this.timeWorkerStart = performance.now();
        this.worker = new Worker(blobUrl);
        this.worker.onmessage = this.syscallHandler.bind(this);
        this.worker.onerror = function (err) {
            if (_this.state === TaskState.Zombie)
                return;
            if (_this.files[2]) {
                console.log(err);
                var msg = new Buffer('Error while executing ' + _this.pendingExePath + ': ' + err.message + '\n', 'utf8');
                _this.files[2].write(msg, -1, function () {
                    setTimeout(function () {
                        _this.kernel.exit(_this, -1);
                    });
                });
            }
            else {
                setTimeout(function () {
                    _this.kernel.exit(_this, -1);
                });
            }
        };
        var heap = this.heap;
        var args = this.forkArgs;
        this.heap = undefined;
        this.forkArgs = undefined;
        this.args = this.pendingArgs;
        this.env = this.pendingEnv;
        this.exePath = this.pendingExePath;
        this.pendingArgs = undefined;
        this.pendingEnv = undefined;
        this.pendingExePath = undefined;
        this.signal('init', [this.args, this.env, this.kernel.debug, this.pid, heap, args], heap ? [heap] : null);
        this.onRunnable(null, this.pid);
        this.onRunnable = undefined;
    };
    Task.prototype.setPriority = function (prio) {
        this.priority += prio;
        if (this.priority < PRIO_MIN)
            this.priority = PRIO_MIN;
        if (this.priority >= PRIO_MAX)
            this.priority = PRIO_MAX - 1;
        return 0;
    };
    Task.prototype.wait4 = function (pid, options, cb) {
        if (pid < -1) {
            console.log('TODO: wait4 with pid < -1');
            cb(-constants.ECHILD);
        }
        if (options) {
            console.log('TODO: non-zero options');
        }
        for (var i = 0; i < this.children.length; i++) {
            var child = this.children[i];
            if (pid !== -1 && pid !== 0 && child.pid !== pid)
                continue;
            if (child.state !== TaskState.Zombie) {
                this.waitQueue.push([pid, options, cb]);
                return;
            }
            var wstatus = (child.exitCode >>> 0) % (1 << 8);
            cb(child.pid, wstatus, null);
            this.kernel.wait(child.pid);
            this.children.splice(i, 1);
            return;
        }
        cb(-constants.ECHILD);
    };
    Task.prototype.childDied = function (pid, code) {
        if (!this.waitQueue.length) {
            this.signal('child', [pid, code, 0]);
            return;
        }
        var queue = this.waitQueue;
        this.waitQueue = [];
        for (var i = 0; i < queue.length; i++) {
            this.wait4.apply(this, queue[i]);
        }
    };
    Task.prototype.signal = function (name, args, transferrable) {
        this.schedule({
            id: -1,
            name: name,
            args: args,
        }, transferrable);
    };
    Task.prototype.schedule = function (msg, transferrable) {
        if (this.state === TaskState.Zombie)
            return;
        this.state = TaskState.Running;
        if (STRACE)
            console.log('[' + this.pid + '|' + msg.id + '] \tCOMPLETE');
        this.worker.postMessage(msg, transferrable || []);
    };
    Task.prototype.exit = function (code) {
        this.state = TaskState.Zombie;
        this.exitCode = code;
        this.onRunnable = undefined;
        this.blobUrl = undefined;
        if (this.timeFirstMsg && false) {
            var exit = performance.now();
        }
        for (var n in this.files) {
            if (!this.files.hasOwnProperty(n))
                continue;
            if (this.files[n]) {
                this.files[n].unref();
                this.files[n] = undefined;
            }
        }
        if (this.worker) {
            this.worker.onmessage = undefined;
            this.worker.terminate();
            this.worker = undefined;
        }
        this.sheap = null;
        this.heapu8 = null;
        this.heap32 = null;
        for (var i = 0; i < this.children.length; i++) {
            var child = this.children[i];
            child.parent = this.parent;
            if (!child.parent && child.state === TaskState.Zombie)
                this.kernel.wait(child.pid);
        }
        if (this.parent)
            this.parent.childDied(this.pid, code);
        if (this.onExit)
            this.onExit(this.pid, this.exitCode);
        if (!this.parent)
            this.kernel.wait(this.pid);
    };
    Task.prototype.nextMsgId = function () {
        return ++this.msgIdSeq;
    };
    Task.prototype.syscallHandler = function (ev) {
        if (ev.data.trap) {
            this.syncSyscallStart = performance.now();
            this.syncSyscall(ev.data.trap, ev.data.args);
            return;
        }
        var syscall = types_1.Syscall.From(this, ev);
        if (!syscall) {
            console.log('bad syscall message, dropping');
            return;
        }
        if (this.state === TaskState.Zombie)
            return;
        this.state = TaskState.Interruptable;
        this.kernel.doSyscall(syscall);
    };
    return Task;
}());
exports.Task = Task;
;
function Boot(fsType, fsArgs, cb, args) {
    var _this = this;
    if (args === void 0) { args = {}; }
    var browserfs = {};
    bfs.install(browserfs);
    Buffer = browserfs.Buffer;
    if (typeof window !== 'undefined' && !window.Buffer) {
        window.Buffer = browserfs.Buffer;
    }
    var rootConstructor = bfs.FileSystem[fsType];
    if (!rootConstructor) {
        setImmediate(cb, 'unknown FileSystem type: ' + fsType);
        return;
    }
    var asyncRoot = new (Function.prototype.bind.apply(rootConstructor, [null].concat(fsArgs)));
    asyncRoot.supportsSynch = function () { return false; };
    function finishInit(root, err) {
        if (err) {
            cb(err, undefined);
            return;
        }
        BootWith(root, cb, args);
    }
    if (args.readOnly) {
        if (asyncRoot.initialize) {
            asyncRoot.initialize(finishInit.bind(this, asyncRoot));
        }
        else {
            finishInit(asyncRoot, null);
        }
    }
    else {
        if (asyncRoot.initialize) {
            asyncRoot.initialize(function (err) {
                if (err) {
                    cb(err, undefined);
                    return;
                }
                var writable = new bfs.FileSystem['InMemory']();
                var overlaid = new bfs.FileSystem['OverlayFS'](writable, asyncRoot);
                overlaid.initialize(finishInit.bind(_this, overlaid));
            });
        }
        else {
            var writable = new bfs.FileSystem['InMemory']();
            var overlaid = new bfs.FileSystem['OverlayFS'](writable, asyncRoot);
            overlaid.initialize(finishInit.bind(this, overlaid));
        }
    }
}
exports.Boot = Boot;
function BootWith(rootFs, cb, args) {
    if (args === void 0) { args = {}; }
    var nCPUs = 1;
    var browserfs = {};
    bfs.install(browserfs);
    Buffer = browserfs.Buffer;
    if (typeof window !== 'undefined' && !window.Buffer) {
        window.Buffer = browserfs.Buffer;
    }
    bfs.initialize(rootFs);
    var fs = bfs.BFSRequire('fs');
    var k = new Kernel(fs, nCPUs, args);
    window.kernel = k;
    setImmediate(cb, null, k);
}
exports.BootWith = BootWith;
function BrowsixFSes() {
    return bfs.FileSystem;
}
exports.BrowsixFSes = BrowsixFSes;
if (typeof window !== 'undefined') {
    window.Boot = Boot;
    window.BootWith = BootWith;
    window.BrowsixFSes = BrowsixFSes;
}
