'use strict';
var constants_1 = require('./constants');
var CUTOFF = 8192;
var Pipe = (function () {
    function Pipe() {
        this.buf = new Buffer(0);
        this.refcount = 1;
        this.readWaiter = undefined;
        this.writeWaiter = undefined;
        this.closed = false;
    }
    Pipe.prototype.write = function (s) {
        var b = new Buffer(s);
        this.writeBuffer(b, function (err, len) { });
    };
    Pipe.prototype.writeBuffer = function (b, cb) {
        this.buf = Buffer.concat([this.buf, b]);
        if (this.readWaiter) {
            var waiter = this.readWaiter;
            this.readWaiter = undefined;
            waiter();
        }
        if (this.buf.length <= CUTOFF) {
            cb(0, b.length);
        }
        else {
            if (this.writeWaiter) {
                console.log('ERROR: expected no other write waiter');
            }
            this.writeWaiter = function () {
                cb(0, b.length);
            };
        }
    };
    Pipe.prototype.read = function (buf, off, len, pos, cb) {
        var _this = this;
        if (this.buf.length || this.closed) {
            var n = this.buf.copy(buf, pos, off, off + len);
            if (this.buf.length === off + n)
                this.buf = new Buffer(0);
            else
                this.buf = this.buf.slice(off + n);
            return cb(undefined, n);
        }
        this.readWaiter = function () {
            var n = _this.buf.copy(buf, pos, off, off + len);
            if (_this.buf.length === off + n)
                _this.buf = new Buffer(0);
            else
                _this.buf = _this.buf.slice(off + n);
            cb(undefined, n);
            if (_this.writeWaiter) {
                var waiter = _this.writeWaiter;
                _this.writeWaiter = undefined;
                waiter();
            }
        };
    };
    Pipe.prototype.readSync = function () {
        return this.buf;
    };
    Pipe.prototype.ref = function () {
        this.refcount++;
    };
    Pipe.prototype.unref = function () {
        this.refcount--;
        if (this.refcount)
            return;
        this.closed = true;
        if (!this.readWaiter)
            return;
        this.readWaiter();
        this.readWaiter = undefined;
    };
    return Pipe;
}());
exports.Pipe = Pipe;
function isPipe(f) {
    return f instanceof PipeFile;
}
exports.isPipe = isPipe;
var PipeFile = (function () {
    function PipeFile(pipe) {
        if (!pipe)
            pipe = new Pipe();
        this.pipe = pipe;
    }
    PipeFile.prototype.addEventListener = function (evName, cb) {
        if (evName !== 'write') {
            console.log('eventListener only available on PipeFile for write');
            return;
        }
        this.writeListener = cb;
    };
    PipeFile.prototype.read = function (buf, pos, cb) {
        if (pos !== -1)
            return cb(-constants_1.ESPIPE);
        this.pipe.read(buf, 0, buf.length, 0, cb);
    };
    PipeFile.prototype.write = function (buf, pos, cb) {
        if (pos !== -1)
            return cb(-constants_1.ESPIPE);
        this.pipe.writeBuffer(buf, cb);
        if (this.writeListener)
            this.writeListener(-1, buf.toString('utf-8'));
    };
    PipeFile.prototype.stat = function (cb) {
        throw new Error('TODO: PipeFile.stat not implemented');
    };
    PipeFile.prototype.llseek = function (offhi, offlo, whence, cb) {
        cb(-constants_1.ESPIPE, undefined);
    };
    PipeFile.prototype.readdir = function (cb) {
        setTimeout(cb, 0, 'cant readdir on normal file');
    };
    PipeFile.prototype.readSync = function () {
        return this.pipe.readSync();
    };
    PipeFile.prototype.ref = function () {
        this.pipe.ref();
    };
    PipeFile.prototype.unref = function () {
        this.pipe.unref();
    };
    return PipeFile;
}());
exports.PipeFile = PipeFile;
