'use strict';
var constants_1 = require('./constants');
var pipe_1 = require('./pipe');
function isSocket(f) {
    return f instanceof SocketFile;
}
exports.isSocket = isSocket;
var SocketFile = (function () {
    function SocketFile(task) {
        this.isListening = false;
        this.parent = undefined;
        this.refCount = 1;
        this.peer = undefined;
        this.outgoing = undefined;
        this.incoming = undefined;
        this.incomingQueue = [];
        this.acceptQueue = [];
        this.task = task;
    }
    SocketFile.prototype.stat = function (cb) {
        throw new Error('TODO: SocketFile.stat not implemented');
    };
    SocketFile.prototype.readdir = function (cb) {
        setTimeout(cb, 0, 'cant readdir on normal file');
    };
    SocketFile.prototype.listen = function (cb) {
        this.isListening = true;
        cb(0);
    };
    SocketFile.prototype.accept = function (cb) {
        if (!this.incomingQueue.length) {
            this.acceptQueue.push(cb);
            return;
        }
        var queued = this.incomingQueue.shift();
        var remote = queued.s;
        var local = new SocketFile(this.task);
        local.addr = queued.addr;
        local.port = queued.port;
        var outgoing = new pipe_1.Pipe();
        var incoming = new pipe_1.Pipe();
        local.outgoing = outgoing;
        remote.incoming = outgoing;
        local.incoming = incoming;
        remote.outgoing = incoming;
        local.peer = remote;
        remote.peer = local;
        cb(0, local, queued.addr, queued.port);
        queued.cb(null);
    };
    SocketFile.prototype.doAccept = function (remote, remoteAddr, remotePort, cb) {
        if (!this.acceptQueue.length) {
            this.incomingQueue.push({
                s: remote,
                addr: remoteAddr,
                port: remotePort,
                cb: cb,
            });
            return;
        }
        var acceptCB = this.acceptQueue.shift();
        var local = new SocketFile(this.task);
        local.addr = remoteAddr;
        local.port = remotePort;
        var outgoing = new pipe_1.Pipe();
        var incoming = new pipe_1.Pipe();
        local.outgoing = outgoing;
        remote.incoming = outgoing;
        local.incoming = incoming;
        remote.outgoing = incoming;
        local.peer = remote;
        remote.peer = local;
        acceptCB(0, local, remoteAddr, remotePort);
        cb(null);
    };
    SocketFile.prototype.connect = function (addr, port, cb) {
        this.task.kernel.connect(this, addr, port, cb);
    };
    SocketFile.prototype.read = function (buf, pos, cb) {
        if (pos !== -1)
            return cb(-constants_1.ESPIPE);
        this.incoming.read(buf, 0, buf.length, undefined, cb);
    };
    SocketFile.prototype.write = function (buf, pos, cb) {
        if (pos !== -1)
            return cb(-constants_1.ESPIPE);
        this.outgoing.writeBuffer(buf, cb);
    };
    SocketFile.prototype.readSync = function () {
        return this.incoming.readSync();
    };
    SocketFile.prototype.llseek = function (offhi, offlo, whence, cb) {
        cb(-constants_1.EINVAL, undefined);
    };
    SocketFile.prototype.ref = function () {
        this.refCount++;
        if (this.outgoing)
            this.outgoing.ref();
        if (this.incoming)
            this.incoming.ref();
    };
    SocketFile.prototype.unref = function () {
        if (this.outgoing)
            this.outgoing.unref();
        if (this.incoming)
            this.incoming.unref();
        this.refCount--;
        if (!this.refCount) {
            if (this.isListening)
                this.task.kernel.unbind(this, this.addr, this.port);
        }
    };
    return SocketFile;
}());
exports.SocketFile = SocketFile;
