'use strict';
var syscall_1 = require('../browser-node/syscall');
var ENOSYS = 38;
var AT_FDCWD = -0x64;
function sys_ni_syscall(cb, trap) {
    console.log('ni syscall ' + trap);
    debugger;
    setTimeout(cb, 0, [-1, 0, -ENOSYS]);
}
function sys_wait4(cb, trap, pid, wstatus, options, rusage) {
    var done = function (pid, wstatusIn, rusage) {
        if (rusage === void 0) { rusage = null; }
        if (pid > 0)
            wstatus.$set(wstatusIn);
        cb([pid, 0, 0]);
    };
    syscall_1.syscall.wait4(pid, options, done);
}
function sys_kill(cb, trap, pid, sig) {
    var done = function (err) {
        cb([err ? -1 : 0, 0, err ? -err : 0]);
    };
    syscall_1.syscall.kill(pid, sig, done);
}
function sys_getpid(cb, trap) {
    var done = function (err, pid) {
        cb([pid, 0, 0]);
    };
    syscall_1.syscall.getpid(done);
}
function sys_getppid(cb, trap) {
    var done = function (err, pid) {
        cb([pid, 0, 0]);
    };
    syscall_1.syscall.getppid(done);
}
var zeroBuf = new Uint8Array(0);
function sys_spawn(cb, trap, dir, argv0, argv, envv, fds) {
    if (!(dir instanceof Uint8Array))
        dir = zeroBuf;
    if (!(argv0 instanceof Uint8Array))
        argv0 = zeroBuf;
    argv = argv.filter(function (x) { return x instanceof Uint8Array; });
    envv = envv.filter(function (x) { return x instanceof Uint8Array; });
    var done = function (err, pid) {
        cb([err ? -1 : pid, 0, err ? err : 0]);
    };
    syscall_1.syscall.spawn(dir, argv0, argv, envv, fds, done);
}
function sys_pipe2(cb, trap, fds, flags) {
    var done = function (err, fd1, fd2) {
        if (!err) {
            fds[0] = fd1;
            fds[1] = fd2;
        }
        cb([err ? err : 0, 0, err ? err : 0]);
    };
    syscall_1.syscall.pipe2(flags, done);
}
function sys_getcwd(cb, trap, path, len) {
    var done = function (p) {
        path.subarray(0, len).set(p);
        var nullPos = p.length;
        if (nullPos >= path.byteLength)
            nullPos = path.byteLength;
        path[nullPos] = 0;
        cb([p.length + 1, 0, 0]);
    };
    syscall_1.syscall.getcwd(done);
}
function sys_chdir(cb, trap, path) {
    var done = function (err) {
        cb([err ? -1 : 0, 0, err ? -err : 0]);
    };
    var len = path.length;
    if (len && path[path.length - 1] === 0)
        len--;
    syscall_1.syscall.chdir(path.subarray(0, len), done);
}
function sys_ioctl(cb, trap, fd, request, argp) {
    var done = function (err, buf) {
        if (!err && argp.byteLength !== undefined)
            argp.set(buf);
        cb([err ? err : buf.byteLength, 0, err ? -1 : 0]);
    };
    syscall_1.syscall.ioctl(fd, request, argp.byteLength, done);
}
function sys_getdents64(cb, trap, fd, buf, len) {
    var done = function (err, dents) {
        if (dents)
            buf.set(dents);
        cb([err, 0, err < 0 ? -1 : 0]);
    };
    syscall_1.syscall.getdents(fd, len, done);
}
function sys_read(cb, trap, fd, readArray, readLen) {
    var done = function (err, dataLen, data) {
        if (!err) {
            for (var i = 0; i < dataLen; i++)
                readArray[i] = data[i];
        }
        cb([dataLen, 0, err ? -1 : 0]);
    };
    syscall_1.syscall.pread(fd, readLen, -1, done);
}
function sys_write(cb, trap, fd, buf, blen) {
    var done = function (err, len) {
        cb([len, 0, err ? -1 : 0]);
    };
    syscall_1.syscall.pwrite(fd, new Uint8Array(buf, 0, blen), -1, done);
}
function sys_stat(cb, trap, path, buf) {
    var done = function (err, stats) {
        if (!err)
            buf.set(stats);
        cb([err ? -1 : 0, 0, err ? -1 : 0]);
    };
    var len = path.length;
    if (len && path[path.length - 1] === 0)
        len--;
    syscall_1.syscall.stat(path.subarray(0, len), done);
}
function sys_lstat(cb, trap, path, buf) {
    var done = function (err, stats) {
        if (!err)
            buf.set(stats);
        cb([err ? -1 : 0, 0, err ? -1 : 0]);
    };
    var len = path.length;
    if (len && path[path.length - 1] === 0)
        len--;
    syscall_1.syscall.lstat(path.subarray(0, len), done);
}
function sys_fstat(cb, trap, fd, buf) {
    var done = function (err, stats) {
        if (!err)
            buf.set(stats);
        cb([err ? -1 : 0, 0, err ? -1 : 0]);
    };
    syscall_1.syscall.fstat(fd, done);
}
function sys_readlinkat(cb, trap, fd, path, buf, blen) {
    if (fd !== AT_FDCWD) {
        console.log('openat: TODO: we only support AT_FDCWD');
        setTimeout(cb, 0, [-1, 0, -1]);
        return;
    }
    var done = function (err, linkString) {
        if (!err)
            buf.set(linkString);
        cb([err ? -1 : linkString.length, 0, err ? -1 : 0]);
    };
    var len = path.length;
    if (len && path[path.length - 1] === 0)
        len--;
    syscall_1.syscall.readlink(path.subarray(0, len), done);
}
function sys_openat(cb, trap, fd, path, flags, mode) {
    fd = fd | 0;
    if (fd !== AT_FDCWD) {
        console.log('openat: TODO: we only support AT_FDCWD');
        setTimeout(cb, 0, [-1, 0, -1]);
        return;
    }
    var done = function (err, fd) {
        cb([fd, 0, err ? -1 : 0]);
    };
    var len = path.length;
    if (len && path[path.length - 1] === 0)
        len--;
    syscall_1.syscall.open(path.subarray(0, len), flags, mode, done);
}
function sys_close(cb, trap, fd) {
    var done = function (err) {
        cb([err ? -1 : 0, 0, err ? -1 : 0]);
    };
    syscall_1.syscall.close(fd, done);
}
function sys_exit_group(cb, trap, code) {
    syscall_1.syscall.exit(code);
}
function sys_socket(cb, trap, domain, type, protocol) {
    var done = function (err, fd) {
        cb([err ? -1 : fd, 0, err ? -1 : 0]);
    };
    syscall_1.syscall.socket(domain, type, protocol, done);
}
function sys_bind(cb, trap, fd, buf, blen) {
    var done = function (err) {
        cb([err ? -1 : 0, 0, err ? -1 : 0]);
    };
    syscall_1.syscall.bind(fd, buf.subarray(0, blen), done);
}
function sys_listen(cb, trap, fd, backlog) {
    var done = function (err) {
        cb([err ? -1 : 0, 0, err ? -1 : 0]);
    };
    syscall_1.syscall.listen(fd, backlog, done);
}
function sys_connect(cb, trap, fd, buf, blen) {
    var done = function (err) {
        cb([err ? -1 : 0, 0, err ? -1 : 0]);
    };
    syscall_1.syscall.connect(fd, buf.subarray(0, blen), done);
}
function sys_getsockname(cb, trap, fd, buf, lenp) {
    var done = function (err, sockInfo) {
        if (!err) {
            buf.set(sockInfo);
            lenp.$set(sockInfo.byteLength);
        }
        cb([err ? -1 : 0, 0, err ? -1 : 0]);
    };
    syscall_1.syscall.getsockname(fd, done);
}
function sys_getpeername(cb, trap, fd, buf, lenp) {
    var done = function (err, sockInfo) {
        if (!err) {
            buf.set(sockInfo);
            lenp.$set(sockInfo.byteLength);
        }
        cb([err ? -1 : 0, 0, err ? -1 : 0]);
    };
    syscall_1.syscall.getpeername(fd, done);
}
function sys_accept4(cb, trap, fd, buf, lenp) {
    var done = function (err, fd, sockInfo) {
        buf.set(sockInfo);
        lenp.$set(sockInfo.length);
        cb([err ? -1 : fd, 0, err ? -1 : 0]);
    };
    syscall_1.syscall.accept(fd, done);
}
function sys_setsockopt(cb, trap) {
    setTimeout(cb, 0, [0, 0, 0]);
}
exports.syscallTbl = [
    sys_read,
    sys_write,
    sys_ni_syscall,
    sys_close,
    sys_stat,
    sys_fstat,
    sys_lstat,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ioctl,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_getpid,
    sys_ni_syscall,
    sys_socket,
    sys_connect,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_bind,
    sys_listen,
    sys_getsockname,
    sys_getpeername,
    sys_ni_syscall,
    sys_setsockopt,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_wait4,
    sys_kill,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_getcwd,
    sys_chdir,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_getppid,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_getdents64,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_exit_group,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_openat,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_readlinkat,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_accept4,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_pipe2,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_ni_syscall,
    sys_spawn,
];
